package ru.mkb.corp.monmod.core;

import MKBCRPAccountInPrognozIO131.AddClientData;
import com.haulmont.cuba.core.TransactionalDataManager;
import com.haulmont.cuba.security.entity.User;
import org.springframework.stereotype.Component;
import ru.mkb.corp.monmod.entity.counterparty.Client;
import ru.mkb.corp.monmod.entity.counterparty.JuridicalClient;
import ru.mkb.corp.monmod.entity.monitoring.ClientSegment;
import ru.mkb.corp.monmod.entity.monitoring.PvLevel;
import ru.mkb.corp.monmod.service.user.UserService;

import javax.inject.Inject;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.Optional;

@Component(CreateJuridicalClient.NAME)
public class CreateJuridicalClient extends ParameterClient implements CreateClient{
    public static final String NAME = "mm_CreateJuridicalClient";
    public static final String CLIENT_MANAGER_ROLE = "manager-client";

    @Inject
    private TransactionalDataManager dataManager;
    @Inject
    private UserService userService;


    @Override
    public Client create(AddClientData addClientData) {
        MKBCRPAccountInPrognozIO131.Client clientData = Optional.ofNullable(
                addClientData.getClient()).orElseThrow(() -> new IllegalArgumentException("Failed to get client"));
        JuridicalClient client = dataManager.load(JuridicalClient.class)
                .query("e.crmId = :crmCode")
                .parameter("crmCode", clientData.getSiebelID())
                .view("juridicalClient-view")
                .optional()
                .orElseGet(() -> dataManager.create(JuridicalClient.class));
        if (!needUpdate(client, addClientData)) {
            return client;
        }
        client.setAbsId(clientData.getClientcode());
        client.setCrmId(clientData.getSiebelID());
        client.setSiebelRowId(clientData.getSiebelRowID());
        client.setSiebelLastUpdateDate(LocalDateTime.now());
        client.setKpp(getKpp(clientData));
        client.setName(clientData.getBrief());
        client.setInn(clientData.getINN());
        client.setFullName(clientData.getName());
        client.setPvLevel(Optional.ofNullable(Integer.parseInt(clientData.getLevelAttention())).map(PvLevel::fromId).orElse(null));
        client.setSegment(Optional.ofNullable(clientData.getSegmenttid()).map(ClientSegment::fromCode).orElse(null));
        client.setCurator(getCurator(clientData));
        User user = userService.createUserByADLogin(clientData.getClientManagerList().getManagerCode().substring(4), Collections.singletonList(CLIENT_MANAGER_ROLE));
        client.setManager(user);
        Client result = dataManager.save(client);
        result.setHolding(getHolding(clientData));
        return dataManager.save(client);
    }

    @Override
    public boolean isProcess(AddClientData addClientData) {
        if(addClientData.getClient().getINN().length() == 10) {
            return true;
        } else return false;
    }

}
