package ru.mkb.corp.monmod.core;

import MKBCRPAccountInPrognozIO131.AddClientData;
import com.haulmont.cuba.core.TransactionalDataManager;
import com.haulmont.cuba.security.entity.User;
import org.springframework.stereotype.Component;
import ru.mkb.corp.monmod.entity.counterparty.Client;
import ru.mkb.corp.monmod.entity.counterparty.SpClient;
import ru.mkb.corp.monmod.entity.monitoring.ClientSegment;
import ru.mkb.corp.monmod.entity.monitoring.PvLevel;
import ru.mkb.corp.monmod.service.user.UserService;


import javax.inject.Inject;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.Optional;

@Component(CreateSpClient.NAME)
public class CreateSpClient extends ParameterClient implements CreateClient{
    public static final String NAME = "mm_CreateSpClient";
    public static final String CLIENT_MANAGER_ROLE = "manager-client";

    @Inject
    TransactionalDataManager dataManager;
    @Inject
    ClientInfoService clientInfoService;
    @Inject
    UserService userService;


    @Override
    public Client create(AddClientData addClientData) {
        MKBCRPAccountInPrognozIO131.Client clientData = Optional.ofNullable(
                addClientData.getClient()).orElseThrow(() -> new IllegalArgumentException("Failed to get client"));
        SpClient client = dataManager.load(SpClient.class)
                .query("e.crmId = :crmCode")
                .parameter("crmCode", clientData.getSiebelID())
                .view("spClient-view")
                .optional()
                .orElseGet(() -> dataManager.create(SpClient.class));
        if (!needUpdate(client, addClientData)) {
            return client;
        }
        client.setName(clientData.getName());
        client.setPersonName(clientData.getFirstName());
        client.setSecondName(clientData.getMiddleName());
        client.setSurname(clientData.getLastName());
        client.setBirth(LocalDate.parse(clientData.getBirthdate(), DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss")));
        client.setAbsId(clientData.getClientcode());
        client.setCrmId(clientData.getSiebelID());
        client.setSiebelRowId(clientData.getSiebelRowID());
        client.setSiebelLastUpdateDate(LocalDateTime.now());
        client.setInn(clientData.getINN());
        client.setSerie(clientInfoService.getSerial(clientData));
        client.setNumber(clientInfoService.getNumber(clientData));
        client.setIsResident(Boolean.valueOf(clientData.getResidentFlag()));
        client.setPvLevel(Optional.ofNullable(Integer.parseInt(clientData.getLevelAttention())).map(PvLevel::fromId).orElse(null));
        client.setSegment(Optional.ofNullable(clientData.getSegmenttid()).map(ClientSegment::fromCode).orElse(null));
        client.setCurator(getCurator(clientData));
        User user = userService.createUserByADLogin(clientData.getClientManagerList().getManagerCode().substring(4), Collections.singletonList(CLIENT_MANAGER_ROLE));
        client.setManager(user);
        return dataManager.save(client);
    }


    @Override
    public boolean isProcess(AddClientData addClientData) {
        if(addClientData.getClient().getINN().length() == 12) {
            return true;
        } else return false;
    }

}
