package ru.mkb.corp.monmod.core;

import MKBCRPAccountInPrognozIO131.AddClientData;
import MKBCRPAccountInPrognozIO131.GroupList;
import com.haulmont.cuba.core.global.DataManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;
import ru.mkb.corp.monmod.entity.counterparty.Client;
import ru.mkb.corp.monmod.entity.counterparty.Curator;
import ru.mkb.corp.monmod.entity.counterparty.Holding;
import ru.mkb.corp.monmod.service.client.CrmHoldingService;
import javax.inject.Inject;
import java.time.LocalDateTime;
import java.util.Optional;

@Slf4j
public abstract class ParameterClient {

    @Inject
    private DataManager dataManager;
    @Inject
    private CuratorListToCuratorMapper curatorListToCuratorMapper;
    @Inject
    private CrmHoldingService crmHoldingService;

    protected Curator getCurator(MKBCRPAccountInPrognozIO131.Client client) {
        Curator theirCurator = curatorListToCuratorMapper.createCuratorFromCuratorsList(client.getCuratorsList());
        if (theirCurator == null) {
            return null;
        }
        return dataManager.load(Curator.class)
                .query("e.xusersid = ?1", theirCurator.getXusersid())
                .optional()
                .orElseGet(() -> dataManager.commit(theirCurator));
    }

    protected Holding getHolding(MKBCRPAccountInPrognozIO131.Client client) {
        return Optional.ofNullable(client.getGroupList())
                .filter(groupLists -> !groupLists.isEmpty())
                .flatMap(groupLists -> groupLists.stream().filter(groupList -> groupList.getGroupType().equals("Холдинг")).findFirst())
                .map(this::getHolding)
                .orElse(null);
    }

    protected Holding getHolding(GroupList groupList) {
        Optional<Holding> ourHolding = dataManager.load(Holding.class)
                .query("e.crmId = :crmCode")
                .parameter("crmCode", groupList.getGroupSiebelID())
                .optional();
        return ourHolding
                .orElseGet(() -> crmHoldingService.createByCrmCode(groupList.getGroupSiebelID()));
    }

    protected String getBik(MKBCRPAccountInPrognozIO131.Client client) {
        return client.getCodeList()
                .stream()
                .filter(codeList -> codeList.getCodeTypeBrief().equals("BANKBIK"))
                .map(codeList -> codeList.getCodeValue())
                .findFirst()
                .orElse(null);
    }

    protected String getKpp(MKBCRPAccountInPrognozIO131.Client client) {
        return client.getCodeList()
                .stream()
                .filter(codeList -> codeList.getCodeTypeBrief().equals("KPPCODE"))
                .map(codeList -> codeList.getCodeValue())
                .findFirst()
                .orElse(null);
    }

    protected boolean needUpdate(Client client, AddClientData addClientData) {
        if (client.getSiebelLastUpdateDate() != null) {
            if (StringUtils.isEmpty(addClientData.getClient().getLastUpdate())) {
                return false;
            } else {
                try {
                    return client.getSiebelLastUpdateDate().compareTo(LocalDateTime.parse(addClientData.getClient().getLastUpdate(), SiebelService.DATE_TIME_FORMATTER)) < 0;
                } catch (Exception e) {
                    log.error("Error while getting AddClientData.client.lastUpdate.", e);
                    return false;
                }
            }
        } else {
            return true;
        }
    }
}
