package ru.mkb.corp.monmod.core;

import org.apache.cxf.headers.Header;
import org.apache.cxf.jaxb.JAXBDataBinding;
import org.springframework.stereotype.Component;
import ru.mkb.corp.monmod.core.config.SiebelConfig;
import ru.mkb.siebelservice.crpallprognozwf.MKBSpcCRPSpcAllSpcPrognozSpcWF;
import ru.mkb.siebelservice.crpallprognozwf.Sblesc11;
import ru.mkb.siebelservice.crpallprognozwf.Sblesc112;
import ru.mkb.siebelservice.crpallprognozwf.Sblesc12;
import ru.mkb.siebelservice.crpallprognozwf.Sblesc13;
import ru.mkb.siebelservice.crpallprognozwf.Sblesc14;
import ru.mkb.siebelservice.crpallprognozwf.Sblesc17;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component(SiebelService.NAME)
public class SiebelService {

    public static final String NAME = "mm_SiebelService";
    public static final String HTTP_SIEBEL_COM_WEBSERVICES_NAMESPACE_URI = "http://siebel.com/webservices";
    private final Sblesc13 clientInfoService;
    private final Sblesc11 findClientService;
    private final Sblesc14 holdingInfoService;
    private final Sblesc12 searchHoldingService;
    private final Sblesc17 affiliatesService;
    private final Sblesc112 spClientInfoService;

    public static final DateTimeFormatter DATE_TIME_FORMATTER = DateTimeFormatter.ofPattern("MM/dd/yyyy HH:mm:ss");

    public SiebelService(SiebelConfig config) {
        MKBSpcCRPSpcAllSpcPrognozSpcWF serviceLocator = new MKBSpcCRPSpcAllSpcPrognozSpcWF();
        Map<String, Object> requestContextMap = new HashMap<>();
        requestContextMap.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, config.getUrl());
        requestContextMap.put(Header.HEADER_LIST, getHeaders(config));
        clientInfoService = serviceLocator.getSblesc13();
        ((BindingProvider) clientInfoService).getRequestContext().putAll(requestContextMap);
        affiliatesService = serviceLocator.getSblesc17();
        ((BindingProvider) affiliatesService).getRequestContext().putAll(requestContextMap);
        spClientInfoService = serviceLocator.getSblesc112();
        ((BindingProvider) spClientInfoService).getRequestContext().putAll(requestContextMap);
        findClientService = serviceLocator.getSblesc11();
        ((BindingProvider) findClientService).getRequestContext().putAll(requestContextMap);
        holdingInfoService = serviceLocator.getSblesc14();
        ((BindingProvider) holdingInfoService).getRequestContext().putAll(requestContextMap);
        searchHoldingService = serviceLocator.getSblesc12();
        ((BindingProvider) searchHoldingService).getRequestContext().putAll(requestContextMap);

    }

    private List<Header> getHeaders(SiebelConfig config) {
        List<Header> headers = new ArrayList<>();
        try {
            QName qNameUsername = new QName(HTTP_SIEBEL_COM_WEBSERVICES_NAMESPACE_URI, "UsernameToken");
            Header usernameHeader = new Header(qNameUsername, config.getUserName(), new JAXBDataBinding(String.class));
            headers.add(usernameHeader);
            QName qNamePaswordText = new QName(HTTP_SIEBEL_COM_WEBSERVICES_NAMESPACE_URI, "PasswordText");
            Header passwordHeader = new Header(qNamePaswordText, config.getPassword(), new JAXBDataBinding(String.class));
            headers.add(passwordHeader);
            QName qNameSessionType = new QName(HTTP_SIEBEL_COM_WEBSERVICES_NAMESPACE_URI, "SessionType");
            Header sessionTypeHeader = new Header(qNameSessionType, "NONE", new JAXBDataBinding(String.class));
            headers.add(sessionTypeHeader);
        } catch (JAXBException e) {
            e.printStackTrace();
        }
        return headers;
    }

    public Sblesc13 getClientInfoService() {
        return clientInfoService;
    }

    public Sblesc11 getFindClientService() {
        return findClientService;
    }

    public Sblesc14 getHoldingInfoService() {
        return holdingInfoService;
    }
    public Sblesc17 getAffiliatesService() {
        return affiliatesService;
    }
    public Sblesc112 getSpClientInfoService() {
        return spClientInfoService;
    }

    public Sblesc12 getSearchHoldingService() {
        return searchHoldingService;
    }
}
