package ru.mkb.corp.monmod.service;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.LoadContext;
import com.haulmont.cuba.core.global.View;
import org.springframework.stereotype.Service;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEventMonitoringCovenant;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEventMonitoringFactor;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.problemfactor.Factor;

import javax.inject.Inject;

@Service(EventMonService.NAME)
public class EventMonServiceBean implements EventMonService {


    @Inject
    private CovEventMonService covEventMonService;
    @Inject
    private FactorEventMonService factorEventMonService;
    @Inject
    private DataManager dataManager;

    @Override
    public void transferEventData(CommonAttributesEvent event) {
        if (event instanceof CovenantEvent)
            transferCovenantEventData(event);
        if (event instanceof ProblemFactorEvent)
            transferFactorEventData(event);
    }
    private void transferCovenantEventData(CommonAttributesEvent event){
        CovenantEvent covEvent = (CovenantEvent) event;
        if (covEvent instanceof FinancialCovenantEvent) {
            covEvent = dataManager.reload(covEvent, "financialCovenantEvent-view");
        } else {
            covEvent = dataManager.reload(covEvent, "nonFinancialCovenantEvent-view");
        }
        covEventMonService.changeCov(covEvent.getCovenant());
    }

    private void transferFactorEventData(CommonAttributesEvent event){
        ProblemFactorEvent factorEvent = (ProblemFactorEvent) event;
        factorEvent = dataManager.reload(factorEvent, "problemFactorEvent-view");
        factorEvent.getFactors().forEach(e -> factorEventMonService.changeFactor(e));
    }
}