package ru.mkb.corp.monmod.service.event;

import com.haulmont.addon.bproc.service.BprocRuntimeService;
import com.haulmont.cuba.core.global.DataManager;
import org.springframework.stereotype.Service;
import ru.mkb.corp.monmod.core.CreateClient;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEvent;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.NonFinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEventMonitoringFactor;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFK;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFKItem;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFP;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFPItem;
import ru.mkb.corp.monmod.entity.event.settings.SettingsNFK;
import ru.mkb.corp.monmod.entity.event.settings.SettingsNFKItem;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.problemfactor.Factor;
import ru.mkb.corp.monmod.entity.problemfactor.MonitoringGraph;
import ru.mkb.corp.monmod.entity.problemfactor.MonitoringPeriodicity;
import ru.mkb.corp.monmod.entity.process.task.TaskProcess;
import ru.mkb.corp.monmod.service.task.TaskService;

import javax.inject.Inject;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service(EventService.NAME)
public class EventServiceBean implements EventService {

    @Inject
    private DataManager dataManager;
    @Inject
    private BprocRuntimeService bprocRuntimeService;
    @Inject
    private List<GeneratedEvent> generatedEvents;


    @Override
    public void generateEvent(MonitoringCard monitoringCard) {
        generatedEvents.forEach(generatedEvent -> generatedEvent.createEvent(monitoringCard));
    }

    @Override
    public void startProcessEvent() {
        List<CommonAttributesEvent> events = dataManager.load(CommonAttributesEvent.class)
                .view("commonAttributesEvent-view")
                .list();
        for (CommonAttributesEvent event : events) {
            if (event.getProcessStartDate().equals(LocalDate.now())) {
                Map<String, Object> processVariables = new HashMap<>();
                processVariables.put("event", event);
                bprocRuntimeService.startProcessInstanceByKey(event.getEventType().getSettings().getProcess(), processVariables);
            }
        }
    }
}