package ru.mkb.corp.monmod.service.event;

import com.haulmont.cuba.core.global.DataManager;
import org.springframework.stereotype.Component;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.event.CovenantEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEventMonitoringCovenant;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.NonFinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.executor.ExecutorEnum;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;

import javax.inject.Inject;
import java.time.LocalDate;

@Component(GeneratedCovenantEvent.NAME)
public class GeneratedCovenantEvent extends GeneratedEvent {
    public static final String NAME = "mm_GeneratedCovenantEvent";

    @Inject
    DataManager dataManager;

    @Override
    public void createEvent(MonitoringCard monitoringCard) {
        for (CovenantMonCard covenantMonCard : monitoringCard.getCovenants()) {
            CovenantEvent event;
            if (covenantMonCard.getCovenant().getFinancialConditionType() != null && covenantMonCard.getCovenant().getFinancialConditionType().equals("Финансовый")) {
                event = dataManager.create(FinancialCovenantEvent.class);
            } else {
                event = dataManager.create(NonFinancialCovenantEvent.class);
            }
            event.setMonitoringCard(monitoringCard);
            LocalDate plannedEndDate = getEndDateEvent(covenantMonCard.getCovenant().getMonitoringGraph(), LocalDate.now());
            LocalDate nextPlannedEndDate = getNextEndDateEvent(covenantMonCard.getCovenant().getMonitoringGraph(), LocalDate.now());
            LocalDate startProcessDate = getProcessStartDate(plannedEndDate, getDaysBeforeCovenantEvent(event, covenantMonCard), covenantMonCard.getCovenant().getMonitoringGraph());
            LocalDate nexStartProcessDate = getProcessStartDate(nextPlannedEndDate, getDaysBeforeCovenantEvent(event, covenantMonCard), covenantMonCard.getCovenant().getMonitoringGraph());
            if (startProcessDate.isAfter(LocalDate.now())) {
                event.setProcessStartDate(startProcessDate);
                event.setStartDate(plannedEndDate);
                event.setPlannedEndDate(plannedEndDate);
            } else {
                event.setProcessStartDate(nexStartProcessDate);
                event.setStartDate(nextPlannedEndDate);
                event.setPlannedEndDate(nextPlannedEndDate);
            }
            event.setCompleted(false);
            dataManager.commit(event);
            createExecutor(event, ExecutorEnum.CONTROL, covenantMonCard.getCovenant().getControlSubdivision().getSubdivision());
            createExecutor(event, ExecutorEnum.INFO, covenantMonCard.getCovenant().getInfoSubdivision().getSubdivision());
            createExecutor(event, ExecutorEnum.CONFIRM, covenantMonCard.getCovenant().getExpertSubdivision().getSubdivision());
            CovenantEventMonitoringCovenant covenant = dataManager.create(CovenantEventMonitoringCovenant.class);
            covenant.setCovenantEvent(event);
            covenant.setCovenantMonCard(covenantMonCard);
            dataManager.commit(covenant);
        }
    }

}
