package ru.mkb.corp.monmod.service.event;

import com.haulmont.cuba.core.global.AppBeans;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.security.entity.Group;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEvent;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.NonFinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.event.executor.Executor;
import ru.mkb.corp.monmod.entity.event.executor.ExecutorEnum;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFK;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFKItem;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFP;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFPItem;
import ru.mkb.corp.monmod.entity.event.settings.SettingsNFK;
import ru.mkb.corp.monmod.entity.event.settings.SettingsNFKItem;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.problemfactor.Factor;
import ru.mkb.corp.monmod.entity.problemfactor.MonitoringGraph;
import ru.mkb.corp.monmod.entity.problemfactor.MonitoringPeriodicity;
import ru.mkb.corp.monmod.service.workdays.WorkDayService;


import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;

public abstract class GeneratedEvent {

    public abstract void createEvent(MonitoringCard monitoringCard);

    public LocalDate getEndDateEvent(MonitoringGraph monitoringGraph, LocalDate localDate) {
        WorkDayService workDayService = AppBeans.get(WorkDayService.class);

        if (monitoringGraph.getMonPeriod() != null) {
            switch (monitoringGraph.getMonPeriod()) {
                case DAILY:
                    localDate = localDate.plusDays(1);
                    break;
                case WEEKLY:
                    localDate = localDate.with(TemporalAdjusters.next(DayOfWeek.FRIDAY));
                    break;
                case MONTHLY:
                    localDate = localDate.plusMonths(1).withDayOfMonth(1);
                    break;
                case QUARTERLY:
                    localDate = localDate.with(localDate.plusMonths(3).getMonth().firstMonthOfQuarter())
                            .with(TemporalAdjusters.firstDayOfMonth());
                    break;
                case ANNUALY:
                    localDate = localDate.plusYears(1).withDayOfYear(1);
                    break;
                case NOTSTANDARD:
                    localDate = localDate.plusMonths(monitoringGraph.getNumOfMon());
                    break;
            }
        }
        return workDayService.getNextWorkDay(localDate);
    }

    public LocalDate getNextEndDateEvent(MonitoringGraph monitoringGraph, LocalDate localDate) {
        WorkDayService workDayService = AppBeans.get(WorkDayService.class);
        if (monitoringGraph.getMonPeriod() != null) {
            switch (monitoringGraph.getMonPeriod()) {
                case DAILY:
                    localDate = localDate.plusDays(2);
                    break;
                case WEEKLY:
                    localDate = localDate.with(TemporalAdjusters.next(DayOfWeek.FRIDAY)).plusWeeks(1);
                    break;
                case MONTHLY:
                    localDate = localDate.plusMonths(2).withDayOfMonth(1);
                    break;
                case QUARTERLY:
                    localDate = localDate.with(localDate.plusMonths(6).getMonth().firstMonthOfQuarter())
                            .with(TemporalAdjusters.firstDayOfMonth());
                    break;
                case ANNUALY:
                    localDate = localDate.plusYears(2).withDayOfYear(1);
                    break;
                case NOTSTANDARD:
                    localDate = localDate.plusMonths(monitoringGraph.getNumOfMon() * 2);
                    break;
            }
        }
        return workDayService.getNextWorkDay(localDate);
    }

    public LocalDate getProcessStartDate(LocalDate endDate, long countNumber, MonitoringGraph monitoringGraph) {
        WorkDayService workDayService = AppBeans.get(WorkDayService.class);
        LocalDate processStartDate;
        if (monitoringGraph.getMonPeriod() != null && monitoringGraph.getMonPeriod().equals(MonitoringPeriodicity.DAILY)){
            processStartDate = endDate;
        } else if (monitoringGraph.getMonPeriod() != null && monitoringGraph.getMonPeriod().equals(MonitoringPeriodicity.WEEKLY)) {
            processStartDate = endDate.with(TemporalAdjusters.previous(DayOfWeek.WEDNESDAY));
        } else processStartDate = workDayService.subtractDaysFromDate(endDate, countNumber);
        return workDayService.getNextWorkDay(processStartDate);
    }

    public long getDaysBeforeCovenantEvent(CovenantEvent event, CovenantMonCard covenant) {
        DataManager dataManager = AppBeans.get(DataManager.class);
        if (event instanceof FinancialCovenantEvent) {
            SettingsFK settingsFK = (SettingsFK) event.getEventType().getSettings();
            settingsFK = dataManager.reload(settingsFK, "settingsFK-view");
            if (covenant.getCovenant().getTypicalCovenant() != null) {
            return settingsFK.getItems().stream()
                    .filter(settingsFKItem -> settingsFKItem.getCovenant().getName().equals(covenant.getCovenant().getTypicalCovenant().getName()))
                    .map(SettingsFKItem::getDays)
                    .findFirst()
                    .orElse(settingsFK.getDays());
            }
        }
        if (event instanceof NonFinancialCovenantEvent) {
            SettingsNFK settingsNFK = (SettingsNFK) event.getEventType().getSettings();
            settingsNFK = dataManager.reload(settingsNFK, "settingsNFK-view");
            if (covenant.getCovenant().getTypicalCovenant() != null) {
            return settingsNFK.getItems().stream()
                    .filter(settingsFKItem -> settingsFKItem.getCovenant().getName().equals(covenant.getCovenant().getTypicalCovenant().getName()))
                    .map(SettingsNFKItem::getDays)
                    .findFirst()
                    .orElse(settingsNFK.getDays());
            }
        }
        return event.getEventType().getSettings().getDays();
    }

    public long getDaysBeforeProblemFactorEvent(ProblemFactorEvent event, Factor factor) {
        DataManager dataManager = AppBeans.get(DataManager.class);
        SettingsFP settingsFP = (SettingsFP) event.getEventType().getSettings();
        settingsFP = dataManager.reload(settingsFP, "settingsFP-view");
        return settingsFP.getItems().stream()
                .filter(settingsFPItem -> settingsFPItem.getProblemFactor().getName().equals(factor.getProblemFactor().getName()))
                .map(SettingsFPItem::getDays)
                .findFirst()
                .orElse(settingsFP.getDays());
    }

    public Executor createExecutor(CommonAttributesEvent event, ExecutorEnum executorEnum, Group group) {
        DataManager dataManager = AppBeans.get(DataManager.class);
        Executor executor = dataManager.create(Executor.class);
        executor.setEvent(event);
        executor.setExecutor(executorEnum);
        executor.setGroup(group);
        return dataManager.commit(executor);
    }
}
