package ru.mkb.corp.monmod.service.event;

import com.haulmont.cuba.core.global.AppBeans;
import com.haulmont.cuba.core.global.DataManager;
import org.springframework.stereotype.Component;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEventMonitoringFactor;
import ru.mkb.corp.monmod.entity.event.executor.Executor;
import ru.mkb.corp.monmod.entity.event.executor.ExecutorEnum;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.problemfactor.Factor;

import javax.inject.Inject;
import java.time.LocalDate;
import java.util.List;
@Component(GeneratedProblemFactorEvent.NAME)
public class GeneratedProblemFactorEvent extends GeneratedEvent {
    public static final String NAME = "mm_GeneratedProblemFactorEvent";

    @Inject
    DataManager dataManager;

    @Override
    public void createEvent(MonitoringCard monitoringCard) {
        for (Factor factor : monitoringCard.getFactors()) {

            List<ProblemFactorEvent> events = dataManager.load(ProblemFactorEvent.class)
                    .query("e.monitoringCard =:monitoringCard")
                    .parameter("monitoringCard", monitoringCard)
                    .view("problemFactorEvent-view")
                    .list();
            ProblemFactorEvent event;
            if (events.isEmpty()) {
                event = createProblemFactorEvent(monitoringCard, factor);
            } else {
                event = events.stream()
                        .flatMap(problemFactorEvent -> problemFactorEvent.getFactors().stream()
                                .filter(problemFactorEventMonitoringFactor ->
                                        problemFactorEventMonitoringFactor.getMonitoringFactor().getSubdivision().equals(factor.getSubdivision()) &&
                                                problemFactorEventMonitoringFactor.getMonitoringFactor().getMonitoringGraph().equals(factor.getMonitoringGraph())))
                        .map(ProblemFactorEventMonitoringFactor::getEvent).findFirst()
                        .orElseGet(() -> createProblemFactorEvent(monitoringCard, factor));
            }
            dataManager.load(Executor.class)
                    .query("e.event =:event")
                    .parameter("event", event)
                    .view("executor-view")
                    .optional()
                    .orElseGet(() -> createExecutor(event, ExecutorEnum.CONTROL, factor.getSubdivision().getSubdivision()));
            ProblemFactorEventMonitoringFactor monitoringFactor = dataManager.create(ProblemFactorEventMonitoringFactor.class);
            monitoringFactor.setMonitoringFactor(factor);
            monitoringFactor.setEvent(event);
            monitoringFactor.setProblemAttributeHolder(factor.getProblemAttributeHolder());
            dataManager.commit(monitoringFactor);
        }
    }

    public ProblemFactorEvent createProblemFactorEvent(MonitoringCard monitoringCard, Factor factor) {
        DataManager dataManager = AppBeans.get(DataManager.class);
        ProblemFactorEvent event = dataManager.create(ProblemFactorEvent.class);
        event.setMonitoringCard(monitoringCard);
        LocalDate plannedEndDate = getEndDateEvent(factor.getMonitoringGraph(), LocalDate.now());
        LocalDate nextPlannedEndDate = getNextEndDateEvent(factor.getMonitoringGraph(), LocalDate.now());
        LocalDate startProcessDate = getProcessStartDate(plannedEndDate, getDaysBeforeProblemFactorEvent(event, factor), factor.getMonitoringGraph());
        LocalDate nexStartProcessDate = getProcessStartDate(nextPlannedEndDate, getDaysBeforeProblemFactorEvent(event, factor), factor.getMonitoringGraph());
        if (startProcessDate.isAfter(LocalDate.now())) {
            event.setProcessStartDate(startProcessDate);
            event.setStartDate(plannedEndDate);
            event.setPlannedEndDate(plannedEndDate);
        } else {
            event.setProcessStartDate(nexStartProcessDate);
            event.setStartDate(nextPlannedEndDate);
            event.setPlannedEndDate(nextPlannedEndDate);
        }
        event.setCompleted(false);
        return dataManager.commit(event);
    }
}
