package ru.mkb.corp.monmod.service.limit;

import com.haulmont.cuba.core.TransactionalDataManager;
import org.springframework.stereotype.Service;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.service.model.uls.LimitComparisonDTO;
import ru.mkb.corp.monmod.service.uls.UlsService;

import java.util.List;
import java.util.stream.Collectors;

@Service(UpdateMonitoringCardService.NAME)
public class UpdateMonitoringCardServiceBean implements UpdateMonitoringCardService {
    private final UlsService ulsService;

    private final TransactionalDataManager txDataManager;

    public UpdateMonitoringCardServiceBean(UlsService ulsService, TransactionalDataManager txDataManager) {
        this.ulsService = ulsService;
        this.txDataManager = txDataManager;
    }

    @Override
    public MonitoringCard updateMonitoringCardFromUls(MonitoringCard monitoringCard) {
        LimitComparisonDTO limitComparisonDTO = ulsService.getUlsInfo(monitoringCard.getClient().getCrmId());
        return setElsData(monitoringCard, limitComparisonDTO);
    }

    @Override
    public void updateMonitoringCardByScheduling() {

        List<MonitoringCard> monitoringCardList = txDataManager.load(MonitoringCard.class)
                .query("select distinct m from mm_MonitoringCard join mm_Holding cm where current_date between m.startDate and m.endDate and m.client=cm")
                .view("monitoringCard-view")
                .list();
        List<MonitoringCard> toSave = monitoringCardList.stream().map(this::updateMonitoringCardFromUls).collect(Collectors.toList());
        for (MonitoringCard card : toSave) {
            txDataManager.save(card);
        }
    }

    private MonitoringCard setElsData(MonitoringCard monitoringCard, LimitComparisonDTO limitComparisonDTO) {
        monitoringCard.setHoldingLimit(limitComparisonDTO.getLimit());
        monitoringCard.setPreviousDebt(limitComparisonDTO.getPast_debt());
        monitoringCard.setCurrentDebt(limitComparisonDTO.getDebt());

        return monitoringCard;
    }
}