package ru.mkb.corp.monmod.service.problemfactor;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.ViewBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttribute;
import ru.mkb.corp.monmod.entity.directories.ProblemFactor;
import ru.mkb.corp.monmod.entity.problemfactorattribute.ProblemFactorAttribute;

import javax.inject.Inject;
import java.util.Collection;
import java.util.Comparator;
import java.util.stream.Collectors;

@Service(ProblemDynamicAttributeService.NAME)
public class ProblemDynamicAttributeServiceBean implements ProblemDynamicAttributeService {

    @Inject
    DataManager dataManager;

    @Override
    public Collection<ProblemFactorAttribute> getAttributesForProblemFactor(ProblemFactor factor) {
        return dataManager.load(ProblemFactorAttribute.class)
                .query("e.factor = :factor")
                .parameter("factor", factor)
                .view(ViewBuilder.of(ProblemFactorAttribute.class)
                        .addView("_local")
                        .add("attribute", "for.cache")
                        .build()
                )
                .list()
                .stream()
                .filter(a -> StringUtils.isNotBlank(a.getAttribute().getCode()))
                .sorted(Comparator.comparing(ProblemFactorAttribute::getOrd))
                .collect(Collectors.toList());
    }
    }