package ru.mkb.corp.monmod.service.rest.api;

import com.haulmont.cuba.core.TransactionalDataManager;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttributeHolder;
import ru.mkb.corp.monmod.entity.covenant.ConditionSpecies;
import ru.mkb.corp.monmod.entity.covenant.ConditionSubspecies;
import ru.mkb.corp.monmod.entity.covenant.ConditionType;
import ru.mkb.corp.monmod.entity.covenant.Covenant;
import ru.mkb.corp.monmod.entity.counterparty.Counterparty;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.limit.ClientType;
import ru.mkb.corp.monmod.entity.limit.Limit;
import ru.mkb.corp.monmod.entity.limit.LimitGraph;
import ru.mkb.corp.monmod.entity.limit.Podlimit;
import ru.mkb.corp.monmod.entity.limit.RelationWithLimit;
import ru.mkb.corp.monmod.entity.limit.RoleClient;
import ru.mkb.corp.monmod.entity.limit.Sublimit;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.service.MonitoringCardService;
import ru.mkb.corp.monmod.service.client.CrmClientService;
import ru.mkb.corp.monmod.service.client.CrmHoldingService;
import ru.mkb.corp.monmod.service.rest.api.dto.ClientDTO;
import ru.mkb.corp.monmod.service.rest.api.dto.ClientRole;
import ru.mkb.corp.monmod.service.rest.api.dto.CovenantDTO;
import ru.mkb.corp.monmod.service.rest.api.dto.LimitData;
import ru.mkb.corp.monmod.service.rest.api.dto.LimitGraphDTO;
import ru.mkb.corp.monmod.service.rest.api.dto.PodlimitToMonmodDTO;
import ru.mkb.corp.monmod.service.rest.api.dto.SublimitToMonmodDTO;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;


@Service(RestApiMonitoringCardService.NAME)
@Transactional
public class RestApiMonitoringCardServiceBean implements RestApiMonitoringCardService {


    @Inject
    private TransactionalDataManager txDataManager;

    @Inject
    private CrmClientService crmClientService;

    @Inject
    private CrmHoldingService crmHoldingService;
    @Inject
    private MonitoringCardService monitoringCardService;


    @Override
    public void createByLimit(LimitData limitData) {
        Counterparty client = null;
        if (limitData.getLimit().getClient().getOrganizationTypeIdent().equals("GROUP_OF_COMPANIES")) {
            client = crmHoldingService.getOrCreateByCrmCode(limitData.getLimit().getClient().getClientCrmCode());
        } else {
            client = crmClientService.getOrCreateByCrmCode(limitData.getLimit().getClient().getClientCrmCode());
        }

        // Создание карточки мониторинга и лимита по клиенту
        if (client != null) {
            Limit limit = getOrCreateLimit(client);
            updateLimit(limit, limitData, client);
            MonitoringCard monitoringCard = getOrCreateMonCard(client);
            RelationWithLimit relation = createRelation(monitoringCard, limit);
            updateRelation(limitData.getLimit().getClient().getClientRole(), relation);
            List<Covenant> covenants = createCovenantList(limitData.getCovenants());
            for (Covenant covenant : covenants) {
                getOrCreateCovenantMonCard(monitoringCard, covenant);
            }


//            Создание карточки мониторинга и лимита по участникам лимита
            List<ClientDTO> participants = limitData.getLimit().getListOfLimitParticipants();
            for (ClientDTO clientDTO : participants) {
                Counterparty participant = crmClientService.getOrCreateByCrmCode(clientDTO.getClientCrmCode());
                MonitoringCard participantMonCard = getOrCreateMonCard(participant);
                RelationWithLimit partRelation = createRelation(participantMonCard, limit);
                updateRelation(clientDTO.getClientRole(), partRelation);
                for (Covenant covenant : covenants) {
                    getOrCreateCovenantMonCard(participantMonCard, covenant);
                }
            }
        }

    }

    @Override
    public Limit updateLimit(Limit limit, LimitData limitData, Counterparty client) {
        limit.setClient(client);
        limit.setLimitID(limitData.getLimit().getLimitID());
        List<LimitGraph> limitGraphs = new ArrayList<>();
        List<Counterparty> clients = new ArrayList<>();
        List<Sublimit> sublimits = new ArrayList<>();
//  Ковенанты лимита
        limit.setCovenants(createCovenantList(limitData.getCovenants()));
//  Получение графика лимитов
        List<LimitGraphDTO> dtoLimitGraphs = limitData.getLimit().getLimitGraphs();
        for (LimitGraphDTO dtoLimitGraph : dtoLimitGraphs) {
            LimitGraph limitGraph = txDataManager.create(LimitGraph.class);
            limitGraph.setStartDate(dtoLimitGraph.getStartDate());
            limitGraph.setEndDate(dtoLimitGraph.getEndDate());
            limitGraph.setAmount(dtoLimitGraph.getAmount());
            limitGraphs.add(limitGraph);
            txDataManager.save(limitGraph);
        }
        limit.setLimitGraphs(limitGraphs);
//  Получение списка участников
        List<ClientDTO> dtoClients = limitData.getLimit().getListOfLimitParticipants();
        for (ClientDTO dtoClient : dtoClients) {
            Counterparty counterparty = null;
            counterparty = crmClientService.getOrCreateByCrmCode(dtoClient.getClientCrmCode());
            clients.add(counterparty);
        }
        limit.setClients(clients);

//  Получение списка сублимитов
        List<SublimitToMonmodDTO> dtoSublimits = limitData.getLimit().getSublimits();
        for (SublimitToMonmodDTO dtoSublimit : dtoSublimits) {
            Sublimit sublimit = txDataManager.create(Sublimit.class);
            List<Podlimit> podlimits = new ArrayList<>();
            List<LimitGraph> sublimitGraphs = new ArrayList<>();
            // Получение списка графика сублимитов
            List<LimitGraphDTO> dtoSublimitGraphs = dtoSublimit.getSublimitGraphs();
            for (LimitGraphDTO dtoSublimitGraph : dtoSublimitGraphs) {
                LimitGraph sublimitGraph = txDataManager.create(LimitGraph.class);
                sublimitGraph.setStartDate(dtoSublimitGraph.getStartDate());
                sublimitGraph.setEndDate(dtoSublimitGraph.getEndDate());
                sublimitGraph.setAmount(dtoSublimitGraph.getAmount());
                sublimitGraphs.add(sublimitGraph);
                txDataManager.save(sublimitGraph);
            }
            //  Получение списка подлимитов из сублимита
            List<PodlimitToMonmodDTO> dtoPodlimits = dtoSublimit.getPodlimits();
            for (PodlimitToMonmodDTO dtoPodlimit : dtoPodlimits) {
                Podlimit podlimit = txDataManager.create(Podlimit.class);
                List<LimitGraph> podlimitGraphs = new ArrayList<>();
                List<Counterparty> podlimitCliens = new ArrayList<>();
//                  Получение графика подлимитов
                List<LimitGraphDTO> dtoPodlimitGraphs = dtoPodlimit.getPodlimitGraphs();
                for (LimitGraphDTO dtoPodlimitGraph : dtoPodlimitGraphs) {
                    LimitGraph podlimitGraph = txDataManager.create(LimitGraph.class);
                    podlimitGraph.setStartDate(dtoPodlimitGraph.getStartDate());
                    podlimitGraph.setEndDate(dtoPodlimitGraph.getEndDate());
                    podlimitGraph.setAmount(dtoPodlimitGraph.getAmount());
                    podlimitGraphs.add(podlimitGraph);
                    txDataManager.save(podlimitGraph);
                }
                // Получение списка участников подлимита
                List<ClientDTO> dtoPodlimitClients = dtoPodlimit.getPodlimitParticipant();
                for (ClientDTO dtoPodlimitClient : dtoPodlimitClients) {
                    Counterparty counterparty = null;
                    counterparty = crmClientService.getOrCreateByCrmCode(dtoPodlimitClient.getClientCrmCode());
                    podlimitCliens.add(counterparty);
                }
                podlimit.setPodlimitID(dtoPodlimit.getPodlimitID());
                podlimit.setClients(podlimitCliens);
                podlimit.setPodlimitGraphs(podlimitGraphs);
                podlimit.setCovenants(createCovenantList(dtoPodlimit.getCovenants()));
                podlimits.add(podlimit);
                txDataManager.save(podlimit);
            }
            sublimit.setSublimitID(dtoSublimit.getSublimitID());
            sublimit.setSublimitName(dtoSublimit.getSublimitName());
            sublimit.setSublimitGraphs(sublimitGraphs);
            sublimit.setPodlimits(podlimits);
            sublimits.add(sublimit);
            txDataManager.save(sublimit);
        }
        limit.setSublimits(sublimits);

        return txDataManager.save(limit);
    }

    @Override
    public MonitoringCard createMonCard(Counterparty client) {
        MonitoringCard monitoringCard = txDataManager.create(MonitoringCard.class);
        monitoringCard.setClient(client);
        List<RespEmployee> respEmployees = monitoringCardService.getRespEmp();
        monitoringCard.setRespEmp(respEmployees);
        for (RespEmployee respEmployee : respEmployees) {
            if (respEmployee.getSubdivision().getName().equals("ОМКП")) {
                monitoringCard.setMonManager(respEmployee.getUser());
            }
        }
        ProblemAttributeHolder problemAttributeHolder = txDataManager.create(ProblemAttributeHolder.class);
        txDataManager.save(problemAttributeHolder);
        monitoringCard.setProblemAttributeHolder(problemAttributeHolder);
        return monitoringCard;
    }

    public List<Covenant> createCovenantList(List<CovenantDTO> covenantDTOS) {
        List<Covenant> covenants = new ArrayList<>();
        for (CovenantDTO covenantDTO : covenantDTOS) {
            Covenant covenant = txDataManager.create(Covenant.class);
            List<Counterparty> clientsListOfCondition = new ArrayList<>();
            covenant.setConditionID(covenantDTO.getConditionID());
            covenant.setFinancialConditionType(covenantDTO.getFinancialConditionType());
            covenant.setDecisionStatement(covenantDTO.getDecisionStatement());
            // Тип ковенанта
            if (covenantDTO.getConditionType().equals("typical")) {
                covenant.setConditionType(ConditionType.TYPICAL);
            } else covenant.setConditionType(ConditionType.ATYPICAL);
            // Вид условия
            if (covenantDTO.getConditionSpecies().equals("once")) {
                covenant.setConditionSpecies(ConditionSpecies.ONCE);
            } else covenant.setConditionSpecies(ConditionSpecies.CONST);
            // Подвид условия
            if (covenantDTO.getConditionSubspecies().equals("beforeprovide")) {
                covenant.setConditionSubspecies(ConditionSubspecies.BEFOREPROVIDE);
            } else if (covenantDTO.getConditionSubspecies().equals("beforesign")) {
                covenant.setConditionSubspecies(ConditionSubspecies.BEFORESIGN);
            } else covenant.setConditionSubspecies(ConditionSubspecies.WITHTERM);

            // Список юридических лиц к которым применяется ковенант
            List<ClientDTO> clientsListOfCovenantDTO = covenantDTO.getClientsListOfCondition();
            for (ClientDTO clientDTO : clientsListOfCovenantDTO) {
                Counterparty counterparty = null;
                if (clientDTO.getOrganizationTypeIdent().equals("GROUP_OF_COMPANIES")) {
                    counterparty = crmHoldingService.getOrCreateByCrmCode(clientDTO.getClientCrmCode());
                } else counterparty = crmClientService.getOrCreateByCrmCode(clientDTO.getClientCrmCode());
                clientsListOfCondition.add(counterparty);
            }
            // Включен в КОД
            covenant.setIncludedToCOD(covenantDTO.isIncludedToCOD());

            txDataManager.save(covenant);
            covenants.add(covenant);
        }
        return covenants;
    }

    public MonitoringCard getOrCreateMonCard(Counterparty client) {
        MonitoringCard monitoringCard = txDataManager.load(MonitoringCard.class)
                .query("e.client = :client")
                .parameter("client", client)
                .view("monitoringCard-view")
                .optional()
                .orElseGet(() -> createMonCard(client));
        return txDataManager.save(monitoringCard);
    }

    public Limit getOrCreateLimit(Counterparty client) {
        return txDataManager.load(Limit.class)
                .query("e.client = :client")
                .parameter("client", client)
                .view("limit-view")
                .optional()
                .orElseGet(() -> txDataManager.create(Limit.class));
    }

    public RelationWithLimit getOrCreateRelation(MonitoringCard monitoringCard, Limit limit) {
        return txDataManager.load(RelationWithLimit.class)
                .query("e.monCard = :monCard and e.limit = :limit")
                .parameter("monCard", monitoringCard)
                .parameter("limit", limit)
                .view("relationWithLimit-view")
                .optional()
                .orElseGet(() -> txDataManager.create(RelationWithLimit.class));
    }

    public RelationWithLimit createRelation(MonitoringCard monitoringCard, Limit limit) {
        RelationWithLimit relation = getOrCreateRelation(monitoringCard, limit);
        relation.setLimit(limit);
        relation.setMonCard(monitoringCard);
        return txDataManager.save(relation);
    }

    public void updateRelation(List<ClientRole> clientRoles, RelationWithLimit relation) {
        for (ClientRole clientRole : clientRoles) {
            RoleClient roleClient = txDataManager.create(RoleClient.class);
            switch (clientRole) {
                case CLIENT:
                    roleClient.setClientType(ClientType.CLIENT);
                    break;
                case BORROWER:
                    roleClient.setClientType(ClientType.BORROWER);
                    break;
                case PLEDGER:
                    roleClient.setClientType(ClientType.PLEDGER);
                    break;
                case GUARANTOR:
                    roleClient.setClientType(ClientType.GUARANTOR);
                    break;
                case BENEFICIARY:
                    roleClient.setClientType(ClientType.BENEFICIARY);
                    break;
                case OTHER:
                    roleClient.setClientType(ClientType.OTHER);
            }
            roleClient.setRelation(relation);
            txDataManager.save(roleClient);
        }
    }
    public CovenantMonCard getOrCreateCovenantMonCard(MonitoringCard monitoringCard, Covenant covenant) {
        CovenantMonCard covenantMonCard =  txDataManager.load(CovenantMonCard.class)
                .query("e.monCard = :monCard and e.covenant = :covenant")
                .parameter("monCard", monitoringCard)
                .parameter("covenant", covenant)
                .view("covenantMonCard-view")
                .optional()
                .orElseGet(() -> createCovenantMonCard(monitoringCard, covenant));
        return txDataManager.save(covenantMonCard);
    }

    public CovenantMonCard createCovenantMonCard(MonitoringCard monitoringCard, Covenant covenant) {
        CovenantMonCard covenantMonCard = txDataManager.create(CovenantMonCard.class);
        covenantMonCard.setCovenant(covenant);
        covenantMonCard.setMonCard(monitoringCard);
        return covenantMonCard;
    }
}