package ru.mkb.corp.monmod.service.task;

import com.haulmont.addon.bproc.entity.ProcessInstanceData;
import com.haulmont.addon.bproc.entity.TaskData;
import com.haulmont.addon.bproc.events.ActivityStartedEvent;
import com.haulmont.addon.bproc.events.ProcessStartedEvent;
import com.haulmont.addon.bproc.events.UserTaskCreatedEvent;
import com.haulmont.addon.bproc.service.BprocRuntimeService;
import com.haulmont.cuba.core.EntityManager;
import com.haulmont.cuba.core.app.events.EntityChangedEvent;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.LoadContext;
import com.haulmont.cuba.core.sys.persistence.EntityChangedEventManager;
import com.haulmont.cuba.security.entity.User;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.event.TransactionPhase;
import org.springframework.transaction.event.TransactionalEventListener;
import ru.mkb.corp.monmod.entity.covenant.Covenant;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEventMonitoringCovenant;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.process.task.TaskInfo;
import ru.mkb.corp.monmod.entity.process.task.TaskProcess;

import javax.inject.Inject;
import javax.persistence.EntityManagerFactory;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.List;
import java.util.UUID;

@Component(ProcessAndTaskListener.NAME)
public class ProcessAndTaskListener {
    public static final String NAME = "mm_ProcessAndTaskListener";
    @Inject
    private DataManager dataManager;
    @Inject
    private BprocRuntimeService bprocRuntimeService;
    @Inject
    private EntityChangedEventManager entityChangedEventManager;

    @EventListener
    public void onTaskCreated(UserTaskCreatedEvent event) {
        TaskData taskData = event.getTaskData();
        User assignedUser = event.getUser();
        createTaskInfo(taskData, assignedUser);
    }

    @EventListener
    public void onProcessStarted(ProcessStartedEvent event) {
        CommonAttributesEvent commonAttributesEvent = (CommonAttributesEvent) bprocRuntimeService.getVariable(event.getProcessInstanceData().getId(), "event");
        TaskProcess taskProcess = dataManager.create(TaskProcess.class);
        taskProcess.setEvent(commonAttributesEvent);
        dataManager.commit(taskProcess);
        bprocRuntimeService.setVariable(event.getProcessInstanceData().getId(), "taskProcess", taskProcess);
    }

    public void createTaskInfo(TaskData taskData, User assignedUser) {
        TaskInfo taskInfo =  dataManager.create(TaskInfo.class);
        taskInfo.setTaskId(taskData.getId());
        taskInfo.setAssignedUser(assignedUser);
        taskInfo.setName(taskData.getName());
        taskInfo.setCreateDate(taskData.getCreateTime().toInstant()
                .atZone(ZoneId.systemDefault()).toLocalDateTime());
        taskInfo.setTaskEnd(false);
        dataManager.commit(taskInfo);
    }

}