package ru.mkb.corp.monmod.service.user;

import com.haulmont.addon.ldap.dto.UserSynchronizationResultDto;
import com.haulmont.addon.ldap.service.UserSynchronizationService;
import com.haulmont.cuba.core.TransactionalDataManager;
import com.haulmont.cuba.core.entity.Entity;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.security.app.role.PredefinedRoleDefinitionRepository;
import com.haulmont.cuba.security.entity.Group;
import com.haulmont.cuba.security.entity.Role;
import com.haulmont.cuba.security.entity.User;
import com.haulmont.cuba.security.entity.UserRole;
import com.haulmont.cuba.security.group.AccessGroupsService;
import com.haulmont.cuba.security.role.RolesService;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service(UserService.NAME)
public class UserServiceBean implements UserService {

    @Inject
    private TransactionalDataManager txDataManager;
    @Inject
    private UserSynchronizationService userSynchronizationService;
    @Inject
    private RolesService rolesService;
    @Inject
    private DataManager dataManager;

    @Override
    public Optional<User> findUserByLogin(String login) {
        return txDataManager.load(User.class).query("lower(e.login) = :login")
                .parameter("login", login.toLowerCase())
                .optional();
    }

    @Override
    public User createUserByADLogin(String adLogin, List<String> roles) {
        UserSynchronizationResultDto userSynchronizationResultDto = userSynchronizationService.synchronizeUser(adLogin, true, null, null, null);
        if (userSynchronizationResultDto.isInactiveUser()) {
            throw new RuntimeException("user is inactive");
        }
        Optional<User> userByLogin = txDataManager.load(User.class)
                .query("lower(e.login) = :login")
                .parameter("login", adLogin.toLowerCase())
                .view("sec-user-view-with-group-roles")
                .optional();
        if (userByLogin.isPresent() && !roles.isEmpty()) {
            final List<UserRole> userRoles = userByLogin.get().getUserRoles();
            final List<UserRole> newUserRoles = new ArrayList<>();
            List<String> roleList = new ArrayList<>(roles);
            txDataManager.load(Role.class)
                    .query("e.name in :names")
                    .parameter("names", roles)
                    .list()
                    .forEach(role -> {
                        roleList.remove(role.getName());
                        if (userRoles.stream().filter(userRole -> userRole.getRole() != null).noneMatch(userRole -> userRole.getRole().equals(role))) {
                            final UserRole userRole = txDataManager.create(UserRole.class);
                            userRole.setRole(role);
                            userRole.setUser(userByLogin.get());
                            userRoles.add(userRole);
                            newUserRoles.add(userRole);
                        }
                    });
            final List<String> collect = userRoles.stream().map(UserRole::getRoleName).collect(Collectors.toList());
            roleList.stream()
                    .filter(s -> !collect.contains(s))
                    .map(s -> rolesService.getRoleDefinitionByName(s))
                    .filter(Objects::nonNull)
                    .forEach(roleDefinition -> {
                        final UserRole userRole = txDataManager.create(UserRole.class);
                        userRole.setRoleDefinition(roleDefinition);
                        userRole.setRoleName(roleDefinition.getName());
                        userRoles.add(userRole);
                        newUserRoles.add(userRole);
                    });
            txDataManager.save(Stream.concat(newUserRoles.stream(), Stream.of(userByLogin.get())).toArray(Entity[]::new));
        }
        return findUserByLogin(adLogin).orElse(null);
    }

    @Override
    public List<User> getUsersByRole(String roleName) {
        return dataManager.load(User.class)
                .query("select distinct ur.user from sec$UserRole ur where ur.role.name =:roleName")
                .parameter("roleName", roleName)
                .list();
    }

    @Override
    public boolean isLeader(User currentUser) {
        for (User user : getUsersByRole("lead")) {
            if (currentUser.equals(user)) return true;
        }
        return false;
    }

    @Override
    public boolean isMonManager(User currentUser) {
        for (User user : getUsersByRole("monManager")) {
            if (currentUser.equals(user)) return true;
        }
        return false;
    }
}