package ru.mkb.corp.monmod.service.workdays;

import com.haulmont.cuba.core.global.Resources;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.springframework.stereotype.Service;
import ru.cbm.WorkingDaysCalculator;
import ru.cbm.WorkingDaysCalculatorFactory;
import ru.mkb.corp.monmod.config.workdays.WorkDaysConfig;

import javax.inject.Inject;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.time.LocalDate;

@Service(WorkDayService.NAME)
@RequiredArgsConstructor
public class WorkDayServiceBean implements WorkDayService {

    private final WorkDaysConfig workDaysConfig;
    private final Resources resources;
    private static final Logger log = org.slf4j.LoggerFactory.getLogger(WorkDayServiceBean.class);
    private final WorkingDaysCalculator workingDaysCalculator;

    @Inject
    public WorkDayServiceBean(WorkDaysConfig workDaysConfig, Resources resources) {
        this.workDaysConfig = workDaysConfig;
        this.resources = resources;
        this.workingDaysCalculator = getWorkDayCalculator();
    }

    public WorkingDaysCalculator getWorkDayCalculator() {
        try (InputStream stream = resources.getResourceAsStream(workDaysConfig.getUri())) {
            File tempFile = File.createTempFile("temp", "file");
            if(stream != null) {
            Files.copy(stream, tempFile.toPath(), StandardCopyOption.REPLACE_EXISTING);
            }
            return WorkingDaysCalculatorFactory.getWorkingDaysCalculator(tempFile.toURI());
        } catch (IOException e) {
            log.error("Error during load date file", e);
            throw new RuntimeException("Can not load working date file");
        }

    }

    @Override
    public boolean isHoliday(LocalDate date) {
        return workingDaysCalculator.isHoliday(date);
    }

    @Override
    public LocalDate getNextWorkDay(LocalDate date) {
        LocalDate checkDate = date;
        while (workingDaysCalculator.isHoliday(checkDate)) {
            checkDate = checkDate.plusDays(1);
        }
        return checkDate;
    }

    @Override
    public LocalDate getPreviousWorkDay(LocalDate date) {
        LocalDate checkDate = date;
        while (workingDaysCalculator.isHoliday(checkDate)) {
            checkDate = checkDate.minusDays(1);
        }
        return checkDate;
    }

    @Override
    public LocalDate addDaysToDate(LocalDate date, long days) {
        return workingDaysCalculator.addWorkingDays(date.atStartOfDay(), days).toLocalDate();
    }

    @Override
    public LocalDate subtractDaysFromDate(LocalDate date, long days) {
        return workingDaysCalculator.subtractWorkingDays(date.atStartOfDay(), days).toLocalDate();
    }
}