package ru.mkb.corp.monmod.entity.counterparty;

import com.haulmont.chile.core.annotations.MetaProperty;
import com.haulmont.chile.core.annotations.NamePattern;
import com.haulmont.cuba.core.global.AppBeans;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.PersistenceHelper;
import ru.mkb.corp.monmod.entity.monitoring.ClientSegment;

import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.util.Optional;

@Table(name = "MM_CLIENT")
@PrimaryKeyJoinColumn(name = "ID")
@Entity(name = "mm_Client")
@DiscriminatorValue("CLIENT")
@NamePattern("%s|name")
public class Client extends Counterparty {
    private static final long serialVersionUID = -7877278756844175083L;

    @Column(name = "FULL_NAME")
    private String fullName;

    @Column(name = "INN")
    private String inn;

    @Column(name = "KPP_")
    private String kpp;

    @JoinColumn(name = "HOLDING_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Holding holding;

    @Column(name = "SEGMENT_ID")
    private String segment;

    @JoinColumn(name = "ORG_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private OrganizationForm organizationForm;

    @Column(name = "IS_RESIDENT")
    private Boolean isResident;

    public Holding getHolding() {
        return holding;
    }

    @Transient
    public String holName = null;

    public void setHolding(Holding holding) {
        this.holding = holding;
        if (holding != null) {
            holName = holding.getName();
        } else {
            holName = "";
        }
    }

    @MetaProperty(related = "holding")
    @Transient
    @Override
    public String getHoldingName() {
        if (PersistenceHelper.isLoaded(this, "holding")) {
            return Optional.ofNullable(holding).map(Counterparty::getName).orElse("");
        } else {
            if (holName == null) {
                holName = AppBeans.get(DataManager.class)
                        .load(Holding.class)
                        .query("select h from mm_Client e join e.holding h where e.id = :id")
                        .parameter("id", this.getId())
                        .optional()
                        .map(Counterparty::getName)
                        .orElse("");
            }
            return holName;
        }
    }

    public String getInn() {
        return inn;
    }

    public void setInn(String inn) {
        this.inn = inn;
    }

    @Transient
    private String cpInn;

    @MetaProperty
    @Transient
    @Override
    public String getCpInn() {
        if (PersistenceHelper.isNew(this) || PersistenceHelper.isLoaded(this, "inn")) {
            return inn;
        } else {
            if (cpInn == null) {
                cpInn = AppBeans.get(DataManager.class)
                        .load(Client.class)
                        .query("select e from mm_Client e where e.id = :id")
                        .parameter("id", this.getId())
                        .optional()
                        .map(Client::getInn)
                        .orElse("");
            }
            return cpInn;
        }
    }

    public String getFullName() {
        return fullName;
    }

    public void setFullName(String fullName) {
        this.fullName = fullName;
    }

    public ClientSegment getSegment() {
        return segment == null ? null : ClientSegment.fromId(segment);
    }

    public void setSegment(ClientSegment clientSeg) {
        this.segment = clientSeg == null ? null : clientSeg.getId();
    }

    public OrganizationForm getOrganizationForm() {
        return organizationForm;
    }

    public void setOrganizationForm(OrganizationForm organizationForm) {
        this.organizationForm = organizationForm;
    }

    public void setSegment(String segment) {
        this.segment = segment;
    }

    public Boolean getIsResident() {
        return isResident;
    }

    public void setIsResident(Boolean resident) {
        isResident = resident;
    }

    @Override
    public String getTypeName() {
        return "";
    }

    public String getKpp() {
        return kpp;
    }

    public void setKpp(String kpp) {
        this.kpp = kpp;
    }
}