package ru.mkb.corp.monmod.entity.covenant;

import com.haulmont.cuba.core.entity.StandardEntity;
import com.haulmont.cuba.core.entity.annotation.EmbeddedParameters;
import lombok.Getter;
import lombok.Setter;
import ru.mkb.corp.monmod.entity.directories.Subdivision;
import ru.mkb.corp.monmod.entity.directories.TypicalCovenant;
import ru.mkb.corp.monmod.entity.counterparty.Counterparty;
import ru.mkb.corp.monmod.entity.problemfactor.MonitoringGraph;

import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import java.util.ArrayList;
import java.util.List;

@Table(name = "MM_COVENANT")
@Entity(name = "mm_Covenant")
@Getter
@Setter
public class Covenant extends StandardEntity {
    private static final long serialVersionUID = -4729583708687974303L;

    @Column(name = "CONDITION_ID")
    private String conditionID;

    @Column(name = "FIN_CONDITION_TYPE")
    private String financialConditionType;

    @Column(name = "CONDITION_TYPE")
    private String conditionType;

    @Column(name = "CONDITION_SPECIES")
    private String conditionSpecies;

    @Column(name = "CONDITION_SUBSPECIES")
    private String conditionSubspecies;

    @Column(name = "DECISION_STATEMENT", length = 4000)
    private String decisionStatement;

    @ManyToMany
    @JoinTable(name = "MM_COVENANT_CLIENTOFCOV",
            joinColumns = @JoinColumn(name = "COVENANT_ID"),
            inverseJoinColumns = @JoinColumn(name = "CLIENT_ID"))
    private List<Counterparty> clientsListOfCondition = new ArrayList<>();

    @Column(name = "INCLUDED_TO_COD")
    private Boolean includedToCOD;

    @ManyToOne
    @JoinColumn(name = "TYPICAL_COVENANT_ID")
    private TypicalCovenant typicalCovenant;

    @EmbeddedParameters(nullAllowed = false)
    @Embedded
    private MonitoringGraph monitoringGraph;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "CONTROL_SUBDIV_ID")
    private Subdivision controlSubdivision;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "INFO_SUBDIV_ID")
    private Subdivision infoSubdivision;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "EXPERT_SUBDIV_ID")
    private Subdivision expertSubdivision;

    public ConditionType getConditionType() {
        return conditionType == null ? null : ConditionType.fromId(conditionType);
    }

    public void setConditionType(ConditionType conditionType) {
        this.conditionType = conditionType == null ? null : conditionType.getId();
    }

    public ConditionSpecies getConditionSpecies() {
        return conditionSpecies == null ? null : ConditionSpecies.fromId(conditionSpecies);
    }

    public void setConditionSpecies(ConditionSpecies conditionSpecies) {
        this.conditionSpecies = conditionSpecies == null ? null : conditionSpecies.getId();
    }

    public ConditionSubspecies getConditionSubspecies() {
        return conditionSubspecies == null ? null : ConditionSubspecies.fromId(conditionSubspecies);
    }

    public void setConditionSubspecies(ConditionSubspecies conditionSubspecies) {
        this.conditionSubspecies = conditionSubspecies == null ? null : conditionSubspecies.getId();
    }
}