package ru.mkb.corp.monmod.entity.event;

import com.haulmont.chile.core.annotations.Composition;
import com.haulmont.chile.core.annotations.MetaProperty;
import com.haulmont.cuba.core.entity.StandardEntity;
import com.haulmont.cuba.core.entity.annotation.OnDelete;
import com.haulmont.cuba.core.global.DeletePolicy;
import lombok.Getter;
import lombok.Setter;
import ru.mkb.corp.monmod.entity.event.executor.Executor;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Inheritance;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.time.LocalDate;
import java.util.List;

@Table(name = "MM_COMMON_ATTRIBUTES_EVENT")
@Entity(name = "mm_CommonAttributesEvent")
@Inheritance(strategy = javax.persistence.InheritanceType.JOINED)
@DiscriminatorColumn(name = "TYPE", discriminatorType = DiscriminatorType.STRING)
@DiscriminatorValue("COMMON")
@Getter
@Setter
public class CommonAttributesEvent extends StandardEntity {
    private static final long serialVersionUID = -8697525584933746889L;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "EVENT_TYPE_ID")
    private EventType eventType;

    @JoinColumn(name = "MONITORING_CARD_ID")
    @OneToOne(fetch = FetchType.LAZY)
    private MonitoringCard monitoringCard;

    @Column(name = "CAPTION")
    private String caption;

    @Column(name = "DESCRIPTION")
    private String description;

    @Column(name = "START_DATE")
    private LocalDate startDate;

    @Column(name = "PLANNED_END_DATE")
    private LocalDate plannedEndDate;

    @Column(name = "FACT_END_DATE")
    private LocalDate factEndDate;

    @Column(name = "PROCESS_START_DATE")
    private LocalDate processStartDate;

    @Column(name = "COMPLETED")
    private Boolean completed;

    @OneToMany(mappedBy = "event")
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    private List<Executor> executors;

    @Transient
    @MetaProperty
    private String styleName;

    @Transient
    @MetaProperty
    private Boolean allDay;

}