package ru.mkb.corp.monmod.entity.monitoring;

import com.haulmont.chile.core.annotations.Composition;
import com.haulmont.cuba.core.entity.StandardEntity;
import com.haulmont.cuba.core.entity.annotation.OnDelete;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.DeletePolicy;
import com.haulmont.cuba.core.global.UserSessionSource;
import com.haulmont.cuba.security.entity.Group;
import com.haulmont.cuba.security.entity.User;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttributeHolder;
import ru.mkb.corp.monmod.entity.covenant.Covenant;
import ru.mkb.corp.monmod.entity.counterparty.Counterparty;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.directories.File;
import ru.mkb.corp.monmod.entity.directories.NoMonitoring;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.limit.RelationWithLimit;
import ru.mkb.corp.monmod.entity.problemfactor.Factor;
import ru.mkb.corp.monmod.service.MonitoringCardService;

import javax.annotation.PostConstruct;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

@Table(name = "MM_MONITORING_CARD")
@Entity(name = "mm_MonitoringCard")
public class MonitoringCard extends StandardEntity {
    private static final long serialVersionUID = 7836981415546559010L;

    @Column(name = "NUMBER_")
    private String number;

    @Column(name = "CREATE_DATE")
    private LocalDate createDate;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "RESP_DEP_ID")
    private Group respDep;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "AUTHOR_ID")
    private User author;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "CLIENT_ID")
    private Counterparty client;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "NO_MONITORING_ID")
    private NoMonitoring noMonitoring;

    @Column(name = "PV_LEVEL", nullable = false)
    @NotNull
    private Integer pvLevel;

    @Column(name = "HAVE_PV")
    private Boolean havePv;

    @Column(name = "CLIENT_IN_DRA")
    private Boolean clientInDra;

    @Column(name = "PROBLEM_CLIENT")
    private Boolean problemClient;

    @Deprecated
    @Column(name = "CLIENT_CLASS")
    private String clientClass;

    @Column(name = "CLIENT_SEG")
    private String clientSeg;

    @Column(name = "LAST_REP_DATE")
    private LocalDate lastRepDate;

    @Column(name = "CLIENT_RATING")
    private String creditRating;

    @Column(name = "START_DATE")
    private LocalDate startDate;

    @Column(name = "END_DATE")
    private LocalDate endDate;

    @Column(name = "CONSTR_PROJ")
    private Boolean constrProj;

    @Column(name = "INVEST_PROJ")
    private Boolean investProj;

    @Column(name = "STRAT_PLAN")
    private Boolean stratPlan;

    @Column(name = "LOMB_CREDIT")
    private Boolean lombCredit;

    @Column(name = "FACTORING")
    private Boolean factoring;

    @Column(name = "ONLY_FACT")
    private Boolean onlyFact;

    @Lob
    @Column(name = "COMMENT_")
    private String comment;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "RES_EMP_ID")
    private User monManager;

    @ManyToMany
    @JoinTable(name = "MM_MON_CARD_RESP_EMP",
            joinColumns = @JoinColumn(name = "MON_CARD_ID"),
            inverseJoinColumns = @JoinColumn(name = "RESP_EMP_ID"))
    private List<RespEmployee> respEmp = new ArrayList<>();

    @OneToMany(mappedBy = "monCard")
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    private List<RelationWithLimit> limits;

    @Column(name = "HOLDING_LIMIT")
    private BigDecimal holdingLimit;

    @Column(name = "CLIENT_LIMIT")
    private BigDecimal clientLimit;

    @Column(name = "PREVIOUS_DEBT")
    private BigDecimal previousDebt;

    @Column(name = "CURRENT_DEBT")
    private BigDecimal currentDebt;

    @Column(name = "START_DATE_LIMIT")
    private LocalDate startDateLimit;

    @Column(name = "END_DATE_LIMIT")
    private LocalDate endDateLimit;

    @Composition
    @OnDelete(DeletePolicy.CASCADE)
    @OneToMany(mappedBy = "monCard")
    private List<Factor> factors;

    @OneToMany(mappedBy = "monitoringCard")
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    private List<File> files;

    @JoinColumn(name = "STRATEGIC_PLAN_ID")
    @OneToOne(fetch = FetchType.LAZY)
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    private StrategicPlan strategicPlan;

    @JoinColumn(name = "TURNOVER_ID")
    @OneToOne(fetch = FetchType.LAZY)
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    private Turnover turnover;

    @OneToMany(mappedBy = "monCard")
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    List<CovenantMonCard> covenants;

    @OneToMany(mappedBy = "monCard")
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    private List<MonitoringCardVersion> monitoringCardVersions;

    @OneToOne(fetch = FetchType.LAZY)
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @JoinColumn(name = "ATTR_HOLDER_ID")
    private ProblemAttributeHolder problemAttributeHolder;

    @JoinColumn(name = "FINANCIAL_POSITION_ID")
    @OneToOne(fetch = FetchType.LAZY)
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    private FinancialPosition financialPosition;

    public List<MonitoringCardVersion> getMonitoringCardVersions() {
        return monitoringCardVersions;
    }

    public void setMonitoringCardVersions(List<MonitoringCardVersion> monitoringCardVersions) {
        this.monitoringCardVersions = monitoringCardVersions;
    }

    public FinancialPosition getFinancialPosition() {
        return financialPosition;
    }

    public void setFinancialPosition(FinancialPosition financialPosition) {
        this.financialPosition = financialPosition;
    }

    public ProblemAttributeHolder getProblemAttributeHolder() {
        return problemAttributeHolder;
    }

    public void setProblemAttributeHolder(ProblemAttributeHolder problemAttributeHolder) {
        this.problemAttributeHolder = problemAttributeHolder;
    }

    public List<CovenantMonCard> getCovenants() {
        return covenants;
    }

    public void setCovenants(List<CovenantMonCard> covenants) {
        this.covenants = covenants;
    }

    public Turnover getTurnover() {
        return turnover;
    }

    public void setTurnover(Turnover turnover) {
        this.turnover = turnover;
    }

    public StrategicPlan getStrategicPlan() {
        return strategicPlan;
    }

    public void setStrategicPlan(StrategicPlan strategicPlan) {
        this.strategicPlan = strategicPlan;
    }

    public List<Factor> getFactors() {
        return factors;
    }

    public void setFactors(List<Factor> factors) {
        this.factors = factors;
    }

    public List<File> getFiles() {
        return files;
    }

    public void setFiles(List<File> files) {
        this.files = files;
    }

    public LocalDate getEndDateLimit() {
        return endDateLimit;
    }

    public void setEndDateLimit(LocalDate endDateLimit) {
        this.endDateLimit = endDateLimit;
    }

    public LocalDate getStartDateLimit() {
        return startDateLimit;
    }

    public void setStartDateLimit(LocalDate startDateLimit) {
        this.startDateLimit = startDateLimit;
    }

    public BigDecimal getCurrentDebt() {
        return currentDebt;
    }

    public void setCurrentDebt(BigDecimal currentDebt) {
        this.currentDebt = currentDebt;
    }

    public BigDecimal getPreviousDebt() {
        return previousDebt;
    }

    public void setPreviousDebt(BigDecimal previousDebt) {
        this.previousDebt = previousDebt;
    }

    public BigDecimal getClientLimit() {
        return clientLimit;
    }

    public void setClientLimit(BigDecimal clientLimit) {
        this.clientLimit = clientLimit;
    }

    public BigDecimal getHoldingLimit() {
        return holdingLimit;
    }

    public void setHoldingLimit(BigDecimal holdingLimit) {
        this.holdingLimit = holdingLimit;
    }

    public List<RelationWithLimit> getLimits() {
        return limits;
    }

    public void setLimits(List<RelationWithLimit> limits) {
        this.limits = limits;
    }

    public List<RespEmployee> getRespEmp() {
        return respEmp;
    }

    public void setRespEmp(List<RespEmployee> respEmp) {
        this.respEmp = respEmp;
    }

    public void setMonManager(User monManager) {
        this.monManager = monManager;
    }

    public User getMonManager() {
        return monManager;
    }


    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public Boolean getOnlyFact() {
        return onlyFact;
    }

    public void setOnlyFact(Boolean onlyFact) {
        this.onlyFact = onlyFact;
    }

    public Boolean getFactoring() {
        return factoring;
    }

    public void setFactoring(Boolean factoring) {
        this.factoring = factoring;
    }

    public Boolean getLombCredit() {
        return lombCredit;
    }

    public void setLombCredit(Boolean lombCredit) {
        this.lombCredit = lombCredit;
    }

    public Boolean getStratPlan() {
        return stratPlan;
    }

    public void setStratPlan(Boolean stratPlan) {
        this.stratPlan = stratPlan;
    }

    public Boolean getInvestProj() {
        return investProj;
    }

    public void setInvestProj(Boolean investProj) {
        this.investProj = investProj;
    }

    public Boolean getConstrProj() {
        return constrProj;
    }

    public void setConstrProj(Boolean constrProj) {
        this.constrProj = constrProj;
    }

    public LocalDate getEndDate() {
        return endDate;
    }

    public void setEndDate(LocalDate endDate) {
        this.endDate = endDate;
    }

    public LocalDate getStartDate() {
        return startDate;
    }

    public void setStartDate(LocalDate startDate) {
        this.startDate = startDate;
    }

    public CreditRating getCreditRating() {
        return creditRating == null ? null : CreditRating.fromId(creditRating);
    }

    public void setCreditRating(CreditRating creditRating) {
        this.creditRating = creditRating == null ? null : creditRating.getId();
    }

    public LocalDate getLastRepDate() {
        return lastRepDate;
    }

    public void setLastRepDate(LocalDate lastRepDate) {
        this.lastRepDate = lastRepDate;
    }

    public Boolean getProblemClient() {
        return problemClient;
    }

    public void setProblemClient(Boolean problemClient) {
        this.problemClient = problemClient;
    }

    public Boolean getClientInDra() {
        return clientInDra;
    }

    public void setClientInDra(Boolean clientInDra) {
        this.clientInDra = clientInDra;
    }

    public ClientSegment getClientSeg() {
        return clientSeg == null ? null : ClientSegment.fromId(clientSeg);
    }

    public void setClientSeg(ClientSegment clientSeg) {
        this.clientSeg = clientSeg == null ? null : clientSeg.getId();
    }

    public Group getRespDep() {
        return respDep;
    }

    public void setRespDep(Group respDep) {
        this.respDep = respDep;
    }

    public Counterparty getClient() {
        return client;
    }

    public void setClient(Counterparty client) {
        this.client = client;
    }

    public NoMonitoring getNoMonitoring() {
        return noMonitoring;
    }

    public void setNoMonitoring(NoMonitoring noMonitoring) {
        this.noMonitoring = noMonitoring;
    }

    public ClientClass getClientClass() {
        return clientClass == null ? null : ClientClass.fromId(clientClass);
    }

    public void setClientClass(ClientClass clientClass) {
        this.clientClass = clientClass == null ? null : clientClass.getId();
    }

    public Boolean getHavePv() {
        return havePv;
    }

    public void setHavePv(Boolean havePv) {
        this.havePv = havePv;
    }

    public PvLevel getPvLevel() {
        return pvLevel == null ? null : PvLevel.fromId(pvLevel);
    }

    public void setPvLevel(PvLevel pvLevel) {
        this.pvLevel = pvLevel == null ? null : pvLevel.getId();
    }

    public User getAuthor() {
        return author;
    }

    public void setAuthor(User author) {
        this.author = author;
    }

    public LocalDate getCreateDate() {
        return createDate;
    }

    public void setCreateDate(LocalDate createDate) {
        this.createDate = createDate;
    }

    public String getNumber() {
        return number;
    }

    public void setNumber(String number) {
        this.number = number;
    }

    @PostConstruct
    public void postConstruct(MonitoringCardService monitoringCardService, UserSessionSource userSessionSource, DataManager dataManager) {
        setNumber("СКМ/" + monitoringCardService.getNextNumber());
        setCreateDate(LocalDate.now());
        setAuthor(userSessionSource.getUserSession().getCurrentOrSubstitutedUser());
        setPvLevel(PvLevel.NONE);
        setStartDate(LocalDate.now());
    }
}