package ru.mkb.corp.monmod.entity.problemfactor;

import com.haulmont.chile.core.annotations.Composition;
import com.haulmont.chile.core.annotations.NamePattern;
import com.haulmont.cuba.core.entity.StandardEntity;
import com.haulmont.cuba.core.entity.annotation.EmbeddedParameters;
import com.haulmont.cuba.core.entity.annotation.OnDelete;
import com.haulmont.cuba.core.entity.annotation.OnDeleteInverse;
import com.haulmont.cuba.core.entity.annotation.PublishEntityChangedEvents;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.DeletePolicy;
import com.haulmont.cuba.security.entity.User;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttributeHolder;
import ru.mkb.corp.monmod.entity.directories.ProblemFactor;
import ru.mkb.corp.monmod.entity.directories.Subdivision;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;

import javax.annotation.PostConstruct;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import java.time.LocalDate;
import java.util.HashMap;

@PublishEntityChangedEvents
@Table(name = "MM_FACTOR")
@Entity(name = "mm_Factor")
@NamePattern("%s|problemFactor")
public class Factor extends StandardEntity {
    private static final long serialVersionUID = 7851364364878625023L;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "PROBLEM_FACTOR_ID")
    @OnDeleteInverse(DeletePolicy.DENY)
    private ProblemFactor problemFactor;

    @Column(name = "START_DATE_MON")
    private LocalDate startDateMon;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "EXECUTOR_ID")
    private User executor;

    @Column(name = "IS_IDENTIFIED")
    private Boolean isIdentified;

    @Column(name = "DATE_OFDET")
    private LocalDate dateOfdet;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "MON_CARD_ID")
    private MonitoringCard monCard;


    @OneToOne(fetch = FetchType.LAZY)
    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @JoinColumn
    @Deprecated
    private ProblemAttributeHolder problemAttributeHolder;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "SUBDIVISION_ID")
    private Subdivision subdivision;

    @EmbeddedParameters(nullAllowed = false)
    @Embedded
    private MonitoringGraph monitoringGraph;

    public MonitoringGraph getMonitoringGraph() {
        return monitoringGraph;
    }

    public Subdivision getSubdivision() {
        return subdivision;
    }

    public void setSubdivision(Subdivision subdivision) {
        this.subdivision = subdivision;
    }

    public void setMonitoringGraph(MonitoringGraph monitoringGraph) {
        this.monitoringGraph = monitoringGraph;
    }

    public ProblemAttributeHolder getProblemAttributeHolder() {
        return problemAttributeHolder;
    }

    public void setProblemAttributeHolder(ProblemAttributeHolder problemAttributeHolder) {
        this.problemAttributeHolder = problemAttributeHolder;
    }

    public LocalDate getDateOfdet() {
        return dateOfdet;
    }

    public void setDateOfdet(LocalDate dateOfdet) {
        this.dateOfdet = dateOfdet;
    }

    public Boolean getIsIdentified() {
        return isIdentified;
    }

    public void setIsIdentified(Boolean isIdentified) {
        this.isIdentified = isIdentified;
    }

    public User getExecutor() {
        return executor;
    }

    public void setExecutor(User executor) {
        this.executor = executor;
    }

    public LocalDate getStartDateMon() {
        return startDateMon;
    }

    public void setStartDateMon(LocalDate startDateMon) {
        this.startDateMon = startDateMon;
    }

    public ProblemFactor getProblemFactor() {
        return problemAttributeHolder != null ? problemAttributeHolder.getFactor() : problemFactor;
    }

    public void setProblemFactor(ProblemFactor factor) {
        if (problemAttributeHolder != null){
            problemAttributeHolder.setFactor(factor);
        }
        this.problemFactor = factor;
    }

    public MonitoringCard getMonCard() {
        return monCard;
    }

    public void setMonCard(MonitoringCard monCard) {
        this.monCard = monCard;
    }

    @PostConstruct
    public void postConstruct(DataManager dataManager) {
        problemAttributeHolder = dataManager.create(ProblemAttributeHolder.class);
        problemAttributeHolder.setDynamicAttributes(new HashMap<>());
    }
}