package ru.mkb.corp.monmod.service.workdays;

import java.net.URISyntaxException;
import java.time.LocalDate;

public interface WorkDayService {
    String NAME = "mm_WorkDayService";

    /**
     * Определение рабочего/выходного дня.
     *
     * @param date проверяемая дата
     * @return true - выходной день, false - рабочий
     * @throws URISyntaxException URISyntaxException
     */
    boolean isHoliday(LocalDate date) throws URISyntaxException;

    /**
     * Получение следующего рабочего дня.
     *
     * @param date дата
     * @return следующий после запрощенного числа рабочий день
     */
    LocalDate getNextWorkDay(LocalDate date);

    /**
     * Получение предыдущего рабочего дня.
     *
     * @param date дата
     * @return предыдущий после запрашиваемого рабочий день
     */
    LocalDate getPreviousWorkDay(LocalDate date);

    /**
     * Добавление рабочих дней к дате (включая начальный день)
     *
     * @param date дата к которой необходимо добавить дни
     * @param days количество добавляемых дней
     * @return итоговая дата
     */
    LocalDate addDaysToDate(LocalDate date, long days);

    /**
     * Вычетание рабочих дней из даты (включая начальный день)
     *
     * @param date дата из которой необходимо вычесть дни
     * @param days количество вычетаемых дней
     * @return итоговая дата
     */
    LocalDate subtractDaysFromDate(LocalDate date, long days);
}