package ru.mkb.corp.monmod.web.processform.task;

import com.haulmont.addon.bproc.web.processform.OutputVariable;
import com.haulmont.addon.bproc.web.processform.ProcessForm;
import com.haulmont.addon.bproc.web.processform.ProcessFormContext;
import com.haulmont.addon.bproc.web.processform.ProcessVariable;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.gui.Notifications;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.DataGrid;
import com.haulmont.cuba.gui.components.DateField;
import com.haulmont.cuba.gui.components.GroupBoxLayout;
import com.haulmont.cuba.gui.components.LinkButton;
import com.haulmont.cuba.gui.components.RadioButtonGroup;
import com.haulmont.cuba.gui.components.TextArea;
import com.haulmont.cuba.gui.export.ExportDisplay;
import com.haulmont.cuba.gui.model.CollectionPropertyContainer;
import com.haulmont.cuba.gui.screen.EditedEntityContainer;
import com.haulmont.cuba.gui.screen.LoadDataBeforeShow;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.StandardEditor;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.security.entity.User;
import ru.mkb.corp.monmod.entity.directories.File;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEvent;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.NonFinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.event.settings.SettingsNFK;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.process.task.TaskInfo;
import ru.mkb.corp.monmod.entity.process.task.TaskProcess;
import ru.mkb.corp.monmod.entity.process.task.TaskResult;
import ru.mkb.corp.monmod.service.task.TaskService;

import javax.inject.Inject;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@UiController("mm_TaskProcessForm")
@UiDescriptor("task-process-form.xml")
@EditedEntityContainer("taskProcessDc")
@LoadDataBeforeShow
@ProcessForm(
        outputVariables ={
                @OutputVariable(name = "result", type = TaskResult.class)
        }
)
public class TaskProcessForm extends StandardEditor<TaskProcess> {

    @Inject
    @ProcessVariable
    protected DateField<Date> completionDateField;
    @Inject
    @ProcessVariable
    protected DateField<Date> createDateField;
    @Inject
    protected TextArea<String> taskField;
    @Inject
    protected ProcessFormContext processFormContext;
    @Inject
    protected RadioButtonGroup<TaskResult> result;
    @ProcessVariable(name = "taskProcess")
    protected TaskProcess taskProcess;
    @ProcessVariable(name = "textTask")
    protected String textTask;
    @Inject
    protected LinkButton monCardBtn;
    @Inject
    protected LinkButton eventBtn;
    @Inject
    protected ScreenBuilders screenBuilders;

    protected MonitoringCard monitoringCard;

    protected String nameEvent;

    protected List<TaskResult> taskResults = new ArrayList<>();
    @Inject
    protected CollectionPropertyContainer<TaskInfo> taskInfoDc;
    @Inject
    protected DataManager dataManager;
    @Inject
    private TextArea<String> commentArea;
    @Inject
    private DataGrid<File> filesGrid;
    @Inject
    protected GroupBoxLayout resultGroupBox;
    @Inject
    private ExportDisplay exportDisplay;
    @Inject
    private Notifications notifications;

    @Subscribe
    protected void onInit(InitEvent event) {
        setEntityToEdit(taskProcess);
    }

    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {
        createDateField.setValue(processFormContext.getTaskData().getCreateTime());
    }


    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        monitoringCard = getEditedEntity().getEvent().getMonitoringCard();
        monCardBtn.setCaption(monitoringCard.getNumber());
        getNameEvent();
        getTextTaskAndResult();
        taskField.setValue(textTask);
        getTaskResult();
    }


    @Subscribe("completeBtn")
    public void onCompleteBtnClick(Button.ClickEvent event) {
        if (checkFilling()) {
        processFormContext.taskCompletion()
                .addProcessVariable("result",result.getValue())
                .saveInjectedProcessVariables()
                .complete();
        taskComplete();
        closeWithCommit();
        }
    }

    @Subscribe("saveBtn")
    public void onSaveBtnClick(Button.ClickEvent event) {
        saveTaskResult();
        closeWithCommit();
    }

    @Subscribe("monCardBtn")
    public void onMonCardBtnClick(Button.ClickEvent event) {
        screenBuilders.editor(MonitoringCard.class, this)
                .withOpenMode(OpenMode.NEW_TAB)
                .editEntity(getEditedEntity().getEvent().getMonitoringCard())
                .show();
    }

    @Subscribe("eventBtn")
    public void onEventBtnClick(Button.ClickEvent event) {
        screenBuilders.editor(CommonAttributesEvent.class, this)
                .withOpenMode(OpenMode.DIALOG)
                .editEntity(getEditedEntity().getEvent())
                .show();
    }


    public TaskInfo saveTaskResult() {
        TaskInfo taskInfo = dataManager.load(TaskInfo.class)
                .query("e.taskId =:taskId")
                .parameter("taskId", processFormContext.getTaskData().getId())
                .view("taskInfo-view")
                .optional()
                .orElse(dataManager.create(TaskInfo.class));
        if (taskInfo.getTaskId() == null) {
            taskInfo.setTaskId(processFormContext.getTaskData().getId());
        }
        taskInfo.setTaskResult(result.getValue());
        taskInfo.setComment(commentArea.getValue());
        taskInfo.setTaskProcess(getEditedEntity());
        taskInfo.setTaskEnd(false);
        taskInfo.setTextTask(textTask);
        taskInfo.setCreateDate(processFormContext.getTaskData().getCreateTime().toInstant()
                .atZone(ZoneId.systemDefault()).toLocalDateTime());
        taskInfo.setName(processFormContext.getTaskData().getName());
        return dataManager.commit(taskInfo);
    }

    public void getTaskResult() {
        if (taskInfoDc != null) {
            TaskInfo taskInfo = taskInfoDc.getMutableItems()
                    .stream()
                    .filter(task -> task.getTaskId().equals(processFormContext.getTaskData().getId()))
                    .findFirst()
                    .orElse(null);
            if (taskInfo != null) {
                result.setValue(taskInfo.getTaskResult());
                commentArea.setValue(taskInfo.getComment());
            }
        }
    }

    public void taskComplete() {
        TaskInfo taskInfo = saveTaskResult();
        taskInfo.setEndDate(LocalDateTime.now());
        taskInfo.setTaskEnd(true);
        dataManager.commit(taskInfo);
    }

    @Subscribe("filesGrid.downloadAction")
    public void onFilesGridDownload(Action.ActionPerformedEvent event) {
        filesGrid.getSelected().forEach(item -> {
            exportDisplay.show(item.getFileDescriptor());
        });
    }

    public boolean checkFilling() {
        if (result.getValue() == null) {
            notifications.create()
                    .withCaption("Не выбран результат")
                    .withType(Notifications.NotificationType.TRAY)
                    .show();
            return false;
        }
        if (!result.getValue().equals(TaskResult.MON_DONE)) {
            return true;
        }
        CommonAttributesEvent event = getEditedEntity().getEvent();
        CovenantEvent covenantEvent = (CovenantEvent) event;
        if (covenantEvent instanceof FinancialCovenantEvent) {
            covenantEvent = dataManager.reload(covenantEvent, "financialCovenantEvent-view");
        }
        if (covenantEvent instanceof NonFinancialCovenantEvent) {
            covenantEvent = dataManager.reload(covenantEvent, "nonFinancialCovenantEvent-view");
        }
        if (covenantEvent.getCovenant().getPlannedResult() == null) {
            notifications.create()
                    .withCaption("В карточке события заполните поле \"Планируемый результат\"")
                    .withType(Notifications.NotificationType.TRAY)
                    .show();
            return false;
        }
        if (covenantEvent.getCovenant().getActualResult() == null) {
            notifications.create()
                    .withCaption("В карточке события заполните поле \"Фактический результат события\"")
                    .withType(Notifications.NotificationType.TRAY)
                    .show();
            return false;
        }
        if (covenantEvent.getCovenant().getViolated() == null) {
            notifications.create()
                    .withCaption("В карточке события заполните поле \"Условие нарушено?\"")
                    .withType(Notifications.NotificationType.TRAY)
                    .show();
            return false;
        }
       return true;
    }

    public void getNameEvent() {
        CommonAttributesEvent event = getEditedEntity().getEvent();
        if (event instanceof FinancialCovenantEvent) {
            nameEvent = "финансовых ковенант";
        }
        if (event instanceof NonFinancialCovenantEvent) {
            nameEvent = "нефинансовых ковенант";
        }
        if (event instanceof ProblemFactorEvent) {
            nameEvent = "факторов проблемности";
        }
    }

    public void getTextTaskAndResult() {
        textTask = "На вас назначено событие \"Мониторинг выполнения " + nameEvent + " клиента " + monitoringCard.getClient().getName() + ".\" " +
                "Для выполнения мониторинга перейдите в карточку события";
        if (getEditedEntity().getEvent() instanceof ProblemFactorEvent) {
            resultGroupBox.setVisible(false);
            result.setValue(TaskResult.DONE);
        } else {
        taskResults.add(TaskResult.MON_DONE);
        taskResults.add(TaskResult.NEED_INFO);
        taskResults.add(TaskResult.NEED_EXPERT);
        result.setOptionsList(taskResults);
        }
    }

}