package ru.mkb.corp.monmod.web.screens;

import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.UiComponents;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.CheckBox;
import com.haulmont.cuba.gui.components.Component;
import com.haulmont.cuba.gui.components.GroupTable;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.Install;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.mkb.corp.monmod.entity.limit.ClientType;
import ru.mkb.corp.monmod.entity.limit.RelationWithLimit;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;

import javax.inject.Inject;
import java.util.Optional;

@UiController("mm_ClientFragment")
@UiDescriptor("client-fragment.xml")
public class ClientFragment extends ScreenFragment {
    private static final Logger log = LoggerFactory.getLogger(ClientFragment.class);
    @Inject
    private CollectionContainer<RelationWithLimit> relWithLimitDc;
    @Inject
    private CollectionLoader<RelationWithLimit> limitsLoader;
    @Inject
    private InstanceContainer<MonitoringCard> monitoringCardDc;
    @Inject
    private UiComponents uiComponents;
    @Inject
    private ScreenBuilders screenBuilders;
    @Inject
    private GroupTable<RelationWithLimit> relWithLimitTable;

    @Subscribe(target = Target.PARENT_CONTROLLER)
    public void onAfterShow(Screen.AfterShowEvent event) {
        limitsLoader.setParameter("mon_card", monitoringCardDc.getItem());
        limitsLoader.load();
        relWithLimitTable.expandAll();
    }

    @Install(to = "relWithLimitTable.borrower", subject = "columnGenerator")
    private Component relWithLimitTableBorrowerColumnGenerator(RelationWithLimit relationWithLimit) {
        CheckBox checkBox = uiComponents.create(CheckBox.class);
        checkBox.setValue(relationWithLimit.getClientRoles().stream()
                .anyMatch(roleClient -> roleClient.getClientType().equals(ClientType.BORROWER)));
        return checkBox;
    }

    @Install(to = "relWithLimitTable.pledger", subject = "columnGenerator")
    private Component relWithLimitTablePledgerColumnGenerator(RelationWithLimit relationWithLimit) {
        CheckBox checkBox = uiComponents.create(CheckBox.class);
        checkBox.setValue(relationWithLimit.getClientRoles().stream().anyMatch(roleClient -> roleClient.getClientType().equals(ClientType.PLEDGER)));
        return checkBox;
    }

    @Install(to = "relWithLimitTable.guarantor", subject = "columnGenerator")
    private Component relWithLimitTableGuarantorColumnGenerator(RelationWithLimit relationWithLimit) {
        CheckBox checkBox = uiComponents.create(CheckBox.class);
        checkBox.setValue(relationWithLimit.getClientRoles().stream().anyMatch(roleClient -> roleClient.getClientType().equals(ClientType.GUARANTOR)));
        return checkBox;
    }

    @Install(to = "relWithLimitTable.other", subject = "columnGenerator")
    private Component relWithLimitTableOtherColumnGenerator(RelationWithLimit relationWithLimit) {
        CheckBox checkBox = uiComponents.create(CheckBox.class);
        checkBox.setValue(relationWithLimit.getClientRoles().stream().anyMatch(roleClient -> roleClient.getClientType().equals(ClientType.BENEFICIARY) || roleClient.getClientType().equals(ClientType.OTHER)));
        return checkBox;
    }

    @Subscribe("relWithLimitTable.view")
    public void onRelWithLimitTableView(Action.ActionPerformedEvent event) {
        Optional.ofNullable(relWithLimitTable.getSingleSelected())
                .map(RelationWithLimit::getMonCard)
                .filter(monitoringCard -> !monitoringCard.equals(monitoringCardDc.getItem()))
                .ifPresent(monitoringCard ->
                        screenBuilders.editor(MonitoringCard.class, this)
                                .editEntity(relWithLimitTable.getSingleSelected().getMonCard())
                                .withOpenMode(OpenMode.NEW_TAB)
                                .show()
                );
    }

}