package ru.mkb.corp.monmod.web.screens.calendarevent;

import com.haulmont.addon.bproc.entity.TaskData;
import com.haulmont.addon.bproc.web.processform.ProcessFormScreens;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.LoadContext;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.Calendar;
import com.haulmont.cuba.gui.components.calendar.CalendarEvent;
import com.haulmont.cuba.gui.components.calendar.EntityCalendarEvent;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.screen.*;
import com.haulmont.cuba.security.global.UserSession;
import com.vaadin.icons.VaadinIcons;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.EventCalendar;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.NonFinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.process.task.TaskInfo;
import ru.mkb.corp.monmod.service.task.TaskService;
import ru.mkb.corp.monmod.service.user.UserService;

import javax.inject.Inject;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Period;
import java.time.temporal.IsoFields;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@UiController("month-calendar")
@UiDescriptor("month-calendar.xml")
@LoadDataBeforeShow
public class MonthCalendar extends StandardLookup<CommonAttributesEvent> {
    @Inject
    private ScreenBuilders screenBuilders;
    @Inject
    private Calendar<LocalDate> calendar;
    @Inject
    private Button dayBtn;
    @Inject
    private Button monthBtn;
    @Inject
    private Button weekBtn;
    @Inject
    private DataManager dataManager;
    @Inject
    private TaskService taskService;

    private List<TaskData> allTasks = new ArrayList<>();
    @Inject
    private CollectionLoader<EventCalendar> eventCalendarsDl;
    @Inject
    private ProcessFormScreens processFormScreens;
    @Inject
    private UserSession userSession;
    @Inject
    private UserService userService;


    @Subscribe
    public void onInit(InitEvent event) {
        allTasks = taskService.getTaskDataList();
        taskService.checkExist();
    }


    @Subscribe("calendar")
    public void onCalendarCalendarEventClick(Calendar.CalendarEventClickEvent<LocalDate> event) {
        EventCalendar eventCalendar = (EventCalendar) event.getEntity();
        CommonAttributesEvent commonAttributesEvent = eventCalendar.getEvent();
        TaskInfo taskInfo = eventCalendar.getTaskInfo();
        if (commonAttributesEvent instanceof FinancialCovenantEvent) {
            screenBuilders.editor(FinancialCovenantEvent.class, this)
                    .withOpenMode(OpenMode.DIALOG)
                    .editEntity((FinancialCovenantEvent) commonAttributesEvent)
                    .show()
                    .addAfterCloseListener(afterCloseEvent -> {
                        reloadCalendar();
                    });
        }
        if (commonAttributesEvent instanceof NonFinancialCovenantEvent) {
            screenBuilders.editor(NonFinancialCovenantEvent.class, this)
                    .withOpenMode(OpenMode.DIALOG)
                    .editEntity((NonFinancialCovenantEvent) commonAttributesEvent)
                    .show()
                    .addAfterCloseListener(afterCloseEvent -> {
                        reloadCalendar();
                    });
        }
        if (commonAttributesEvent instanceof ProblemFactorEvent) {
            screenBuilders.editor(ProblemFactorEvent.class, this)
                    .withOpenMode(OpenMode.DIALOG)
                    .editEntity((ProblemFactorEvent) commonAttributesEvent)
                    .show()
            .addAfterCloseListener(afterCloseEvent -> {
                reloadCalendar();
            });
            }
        if (!eventCalendar.thisEvent) {
            TaskData taskData = allTasks.stream()
                    .filter(task -> task.getId().equals(taskInfo.getTaskId()))
                    .findFirst()
                    .orElse(null);
            Screen screen = processFormScreens.createTaskProcessForm(taskData, this);
            screen.addAfterCloseListener(afterCloseEvent -> {
                allTasks = taskService.getTaskDataList();
                reloadCalendar();
            });
            screen.show();
        }

    }

    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {
        calendar.setFirstDayOfWeek(java.util.Calendar.MONDAY);
        LocalDate beginCalendarDate = LocalDate.now().withDayOfMonth(1);
        LocalDate endCalendarDate = LocalDate.now().withDayOfMonth(LocalDateTime.now().toLocalDate().lengthOfMonth());
        calendar.setStartDate(beginCalendarDate);
        calendar.setEndDate(endCalendarDate);
        monthBtn.setEnabled(false);
    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        addEvents();

    }

    @Install(to = "eventCalendarsDl", target = Target.DATA_LOADER)
    private List<EventCalendar> eventCalendarsDlLoadDelegate(LoadContext<EventCalendar> loadContext) {
        List<EventCalendar> eventCalendars = new ArrayList<>();
        List<CommonAttributesEvent> events = dataManager.load(CommonAttributesEvent.class)
                .view("commonAttributesEvent-view")
                .list();
        if (Period.between(calendar.getStartDate(), calendar.getEndDate()).getDays() == 6) {
            events = events.stream()
                    .filter(event -> (event.getPlannedEndDate().isAfter(calendar.getStartDate().minusDays(1))
                            && event.getPlannedEndDate().isBefore(calendar.getEndDate().plusDays(1)))
                            || (!event.getCompleted() && event.getPlannedEndDate().isBefore(calendar.getStartDate()))
                            || (event.getCompleted() && event.getPlannedEndDate().isAfter(LocalDate.now())))
                    .collect(Collectors.toList());
        } else if (Period.between(calendar.getStartDate(), calendar.getEndDate()).getDays() > 6) {
            events = events.stream()
                    .filter(event -> (event.getPlannedEndDate().isAfter(calendar.getStartDate().minusDays(8))
                            && event.getPlannedEndDate().isBefore(calendar.getEndDate().plusDays(8)))
                            || (!event.getCompleted() && event.getPlannedEndDate().isBefore(calendar.getStartDate()))
                            || (event.getCompleted() && event.getPlannedEndDate().isAfter(LocalDate.now())))
                    .collect(Collectors.toList());
        } else {
            events = events.stream()
                    .filter(event -> (event.getPlannedEndDate().isEqual(LocalDate.now())
                            || (!event.getCompleted() && event.getPlannedEndDate().isBefore(LocalDate.now())))
                            || (event.getCompleted() && event.getPlannedEndDate().isAfter(LocalDate.now())))
                    .collect(Collectors.toList());
        }

        for(CommonAttributesEvent event : events) {
            if (!userService.isMonManager(userSession.getCurrentOrSubstitutedUser())) {
                RespEmployee respEmployee = event.getMonitoringCard().getRespEmp().stream()
                        .filter(re -> re.getUser().equals(userSession.getCurrentOrSubstitutedUser())
                                || re.getSubdivision().getLeader().equals(userSession.getCurrentOrSubstitutedUser()))
                        .findFirst()
                        .orElse(null);
                if (respEmployee != null) {
                    event = event.getExecutors().stream()
                            .filter(executor -> executor.getGroup().equals(respEmployee.getSubdivision().getSubdivision())
                                    || respEmployee.getName().equals("ОМКП"))
                            .map(executor -> executor.getEvent())
                            .findFirst()
                            .orElse(null);
                }
            }
                if (event != null) {
                    EventCalendar eventCalendar = dataManager.create(EventCalendar.class);
                    eventCalendar.setEvent(event);
                    eventCalendar.setThisEvent(true);
                    eventCalendars.add(eventCalendar);
                }
        }
        List<TaskInfo> tasks = new ArrayList<>();
        if (userService.isLeader(userSession.getCurrentOrSubstitutedUser())) {
            tasks = dataManager.load(TaskInfo.class)
                    .query("e.taskEnd =:taskEnd")
                    .parameter("taskEnd", false)
                    .view("taskInfo-view")
                    .list()
                    .stream()
                    .filter(taskInfo -> taskInfo.getAssignedUser().getGroup().equals(userSession.getCurrentOrSubstitutedUser().getGroup())
                            || taskInfo.getAssignedUser().equals(userSession.getCurrentOrSubstitutedUser()))
                    .collect(Collectors.toList());
        } else {
        tasks = dataManager.load(TaskInfo.class)
                .query("e.taskEnd =:taskEnd and e.assignedUser =:assignedUser")
                .parameter("assignedUser", userSession.getCurrentOrSubstitutedUser())
                .parameter("taskEnd", false)
                .view("taskInfo-view")
                .list();
        }
        tasks.stream()
                .filter(taskInfo -> taskInfo.getCreateDate().toLocalDate().isBefore(calendar.getEndDate().plusDays(8)))
                .collect(Collectors.toList());
        for (TaskInfo task : tasks) {
            EventCalendar eventCalendar = dataManager.create(EventCalendar.class);
            eventCalendar.setTaskInfo(task);
            eventCalendar.setThisEvent(false);
            eventCalendars.add(eventCalendar);
        }
        return eventCalendars;
    }





    public void addEvents() {
        List<CalendarEvent> calendarEvents = calendar.getEventProvider().getEvents();
        for (CalendarEvent event : calendarEvents) {
            EntityCalendarEvent<EventCalendar, LocalDate> even =
                    (EntityCalendarEvent<EventCalendar, LocalDate>) event;
            if (even.getEntity().thisEvent) {
            CommonAttributesEvent ev = even.getEntity().getEvent();
                LocalDate eventDate = ev.getPlannedEndDate();
                String styleName = "";
                String caption = getCaption(ev);
                if (ev.getCompleted()) {
                    styleName = "event-grey";
                    eventDate= ev.getFactEndDate();
                } else if (ev.getPlannedEndDate().isBefore(LocalDate.now())) {
                    styleName = "event-red";
                    eventDate = LocalDate.now();
                } else styleName = "event-blue";

                event.setCaption(new String(Character.toChars(VaadinIcons.CALENDAR.getCodepoint())) + " " + caption);
                event.setDescription(caption);
                event.setStyleName(styleName);
                event.setAllDay(true);
                event.setStart(eventDate);
                event.setEnd(eventDate);
            } else {
                TaskInfo task = even.getEntity().getTaskInfo();
                LocalDate taskDate = task.getCreateDate().toLocalDate();
                String styleName = "";
                if (taskDate.isBefore(LocalDate.now())) {
                    styleName = "event-orange";
                    taskDate = LocalDate.now();
                } else styleName = "event-dark-blue";
                event.setCaption(new String(Character.toChars(VaadinIcons.EDIT.getCodepoint())) + " " + task.getName());
                event.setDescription(task.getName());
                event.setStyleName(styleName);
                event.setAllDay(true);
                event.setStart(taskDate);
                event.setEnd(taskDate);
            }
        }
    }

    String getCaption(CommonAttributesEvent event) {
        String holName = "";
        if (event.getMonitoringCard().getClient().getHoldingName() != null &&
                !event.getMonitoringCard().getClient().getHoldingName().equals(event.getMonitoringCard().getClient().getName())) {
            holName = event.getMonitoringCard().getClient().getHoldingName() + ", ";
        }
        String clientName = event.getMonitoringCard().getClient().getName();
        String eventName = event.getEventType().getShortName();
        return holName + clientName + ", " + eventName;
    }

    @Subscribe("weekBtn")
    public void onWeekBtnClick(Button.ClickEvent event) {
        LocalDate beginCalendarDate = LocalDate.now().with(DayOfWeek.MONDAY);
        LocalDate endCalendarDate = beginCalendarDate.plusDays(6);
        calendar.setStartDate(beginCalendarDate);
        calendar.setEndDate(endCalendarDate);
        weekBtn.setEnabled(false);
        monthBtn.setEnabled(true);
        dayBtn.setEnabled(true);
        reloadCalendar();
    }

    @Subscribe("monthBtn")
    public void onMonthBtnClick(Button.ClickEvent event) {
        LocalDate beginCalendarDate = LocalDate.now().withDayOfMonth(1);
        LocalDate endCalendarDate = LocalDate.now().withDayOfMonth(LocalDateTime.now().toLocalDate().lengthOfMonth());
        calendar.setStartDate(beginCalendarDate);
        calendar.setEndDate(endCalendarDate);
        weekBtn.setEnabled(true);
        monthBtn.setEnabled(false);
        dayBtn.setEnabled(true);
        reloadCalendar();
    }

    @Subscribe("dayBtn")
    public void onDayBtnClick(Button.ClickEvent event) {
        LocalDate calendarDate = LocalDate.now();
        calendar.setStartDate(calendarDate);
        calendar.setEndDate(calendarDate);
        weekBtn.setEnabled(true);
        monthBtn.setEnabled(true);
        dayBtn.setEnabled(false);
        reloadCalendar();
    }

    @Subscribe("calendar")
    public void onCalendarCalendarDateClick(Calendar.CalendarDateClickEvent<LocalDate> event) {
        calendar.setStartDate(event.getDate());
        calendar.setEndDate(event.getDate());
        weekBtn.setEnabled(true);
        monthBtn.setEnabled(true);
        dayBtn.setEnabled(false);
        reloadCalendar();
    }

    @Subscribe("calendar")
    public void onCalendarCalendarWeekClick(Calendar.CalendarWeekClickEvent<LocalDate> event) {
        LocalDate desiredDate = LocalDate.now()
                .with(IsoFields.WEEK_OF_WEEK_BASED_YEAR, event.getWeek() - 1)
                .with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
        calendar.setStartDate(desiredDate);
        calendar.setEndDate(desiredDate.plusDays(6));
        weekBtn.setEnabled(false);
        monthBtn.setEnabled(true);
        dayBtn.setEnabled(true);
        reloadCalendar();
    }

    @Subscribe("previousBtn")
    public void onPreviousBtnClick(Button.ClickEvent event) {
        Period period = Period.between(calendar.getStartDate(), calendar.getEndDate());
        LocalDate newDate = calendar.getStartDate().minusDays(1);
        if (period.getDays() > 7) {
            calendar.setStartDate(newDate.withDayOfMonth(1));
            calendar.setEndDate(newDate);
        } else if (period.getDays() == 0) {
            calendar.setStartDate(calendar.getStartDate().minusDays(1));
            calendar.setEndDate(calendar.getEndDate().minusDays(1));
        } else {
            calendar.setStartDate(newDate.minusDays(6));
            calendar.setEndDate(newDate);
        }
        reloadCalendar();
    }

    @Subscribe("nextBtn")
    public void onNextBtnClick(Button.ClickEvent event) {
        Period period = Period.between(calendar.getStartDate(), calendar.getEndDate());
        LocalDate newDate = calendar.getEndDate().plusDays(1);
        if (period.getDays() > 7) {
            calendar.setStartDate(newDate);
            calendar.setEndDate(newDate.withDayOfMonth(calendar.getStartDate().lengthOfMonth()));
        } else if (period.getDays() == 0) {
            calendar.setStartDate(calendar.getStartDate().plusDays(1));
            calendar.setEndDate(calendar.getEndDate().plusDays(1));
        } else {
            calendar.setStartDate(newDate);
            calendar.setEndDate(newDate.plusDays(6));
        }
        reloadCalendar();
    }


    public void reloadCalendar() {
        LocalDate beginCalendarDate = calendar.getStartDate();
        LocalDate endCalendarDate = calendar.getEndDate();
        eventCalendarsDl.load();
        calendar.setStartDate(beginCalendarDate);
        calendar.setEndDate(endCalendarDate);
        addEvents();
    }
}