package ru.mkb.corp.monmod.web.screens.components.problemattribute;

import com.haulmont.chile.core.model.MetaClass;
import com.haulmont.cuba.core.app.dynamicattributes.DynamicAttributesMetaProperty;
import com.haulmont.cuba.core.entity.BaseGenericIdEntity;
import com.haulmont.cuba.core.entity.Category;
import com.haulmont.cuba.core.entity.CategoryAttribute;
import com.haulmont.cuba.gui.components.Component;
import com.haulmont.cuba.gui.components.Field;
import com.haulmont.cuba.gui.components.Form;
import com.haulmont.cuba.gui.components.HasContextHelp;
import com.haulmont.cuba.gui.meta.PropertyType;
import com.haulmont.cuba.gui.meta.StudioProperty;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.web.gui.components.CompositeDescriptor;
import com.haulmont.cuba.web.gui.components.dynamicattributes.DynamicAttributesPanel;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttribute;
import ru.mkb.corp.monmod.entity.directories.ProblemFactor;
import ru.mkb.corp.monmod.entity.problemfactorattribute.ProblemFactorAttribute;
import ru.mkb.corp.monmod.service.problemfactor.ProblemDynamicAttributeService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@CompositeDescriptor("dynamic-attributes-panel.xml")
public class ProblemDynamicAttributesPanel extends DynamicAttributesPanel {

    public static final String NAME = "problemDynamicAttributesPanel";

    @Inject
    private ProblemDynamicAttributeService problemDynamicAttributes;

    private final Map<DynamicAttributesMetaProperty, ProblemFactorAttribute> map = new HashMap<>();


    public ProblemDynamicAttributesPanel() {
        super();
    }

    @Override
    protected void onCreate(CreateEvent createEvent) {
        super.onCreate(createEvent);
        setCategoryFieldVisible(false);
    }

    @Override
    protected List<DynamicAttributesMetaProperty> getPropertiesFilteredByCategory() {
        map.clear();
        MetaClass metaClass = instanceContainer.getEntityMetaClass();
        Optional<ProblemFactor> factor = Optional.ofNullable(instanceContainer.getItemOrNull()).map(e -> e.getValueEx("factor"));
        if (!factor.isPresent()) {
            return Collections.emptyList();
        }
        List<DynamicAttributesMetaProperty> result = new ArrayList<>();
        for (ProblemFactorAttribute categoryAttribute : problemDynamicAttributes.getAttributesForProblemFactor(factor.get())) {
            ProblemAttribute attribute = categoryAttribute.getAttribute();
            Boolean required = attribute.getRequired();
            attribute.setRequired(required || categoryAttribute.getIsRequired());
            DynamicAttributesMetaProperty e = new DynamicAttributesMetaProperty(metaClass, attribute);
            result.add(e);
            attribute.setRequired(required);
            map.put(e, categoryAttribute);
        }
        return result;

    }

    @Override
    @StudioProperty(name = "dataContainer", type = PropertyType.DATACONTAINER_REF, required = true)
    public void setInstanceContainer(InstanceContainer<BaseGenericIdEntity> instanceContainer) {
        super.setInstanceContainer(instanceContainer);
        this.instanceContainer.addItemPropertyChangeListener(event -> {
            if (event.getProperty().equals("factor")) {
                initPropertiesForm();
            }
        });
    }

    @Override
    protected void onInstanceContainerItemChangeEvent(InstanceContainer.ItemChangeEvent<BaseGenericIdEntity> event) {
        initPropertiesForm();
    }

    @Override
    protected Component generateFieldComponent(DynamicAttributesMetaProperty property) {
        Component component = super.generateFieldComponent(property);
        if (component instanceof Field<?>) {
            ((Field<?>) component).setRequired(property.isMandatory());
            ((Field<?>) component).setRequiredMessage(String.format("Заполните поле \"%s\"", property.getAttribute().getName()));
        }
        return component;
    }

    @Override
    protected void initPropertiesForm() {
        propertiesForm.removeAll();
        List<DynamicAttributesMetaProperty> metaProperties = getPropertiesFilteredByCategory();
        List<Component> fieldList = new ArrayList<>();
        for (DynamicAttributesMetaProperty property : metaProperties) {
            Component formField = generateFieldComponent(property);
            Component value = prepareFieldComponent(formField, property);
            final Optional<String> tooltipOption = Optional.ofNullable(map.get(property))
                    .map(ProblemFactorAttribute::getAttribute)
                    .map(ProblemAttribute::getTooltip)
                    .filter(s -> !s.isEmpty());
            if (tooltipOption.isPresent() && value instanceof HasContextHelp) {
                ((HasContextHelp) value).setContextHelpText(tooltipOption.get());
            }
            fieldList.add(value);
        }
        addFieldsToForm(propertiesForm, fieldList);
        initFieldCaptionWidth(propertiesForm);
    }

    protected void addFieldsToForm(Form newPropertiesForm, List<Component> fields) {
        int propertiesCount = fields.size();
        int rowsPerColumn = getRowsPerColumn(propertiesCount);
        int columnNo = 0;
        int fieldsCount = 0;
        for (Component field : fields) {
            fieldsCount++;
            newPropertiesForm.add(field, columnNo);
            if (fieldsCount % rowsPerColumn == 0) {
                columnNo++;
                newPropertiesForm.setColumns(columnNo + 1);
            }
        }
    }
}
