package ru.mkb.corp.monmod.web.screens.components.problemattribute;

import com.google.common.base.Strings;
import com.haulmont.cuba.gui.GuiDevelopmentException;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.model.ScreenData;
import com.haulmont.cuba.gui.screen.FrameOwner;
import com.haulmont.cuba.gui.screen.UiControllerUtils;
import com.haulmont.cuba.gui.xml.layout.loaders.AbstractComponentLoader;
import org.dom4j.Element;

public class ProblemDynamicAttributesPanelLoader extends AbstractComponentLoader<ProblemDynamicAttributesPanel> {

    @Override
    public void createComponent() {
        resultComponent = factory.create(ProblemDynamicAttributesPanel.NAME);
        loadId(resultComponent, element);
    }

    @Override
    public void loadComponent() {
        loadWidth(resultComponent, element);
        loadHeight(resultComponent, element);

        loadDataContainer(resultComponent, element);
        loadColumnsCount(resultComponent, element);
        loadRowsCount(resultComponent, element);
        loadFieldWidth(resultComponent, element);
        loadFieldCaptionWidth(resultComponent, element);
    }

    protected void loadDataContainer(ProblemDynamicAttributesPanel resultComponent, Element element) {
        String containerId = element.attributeValue("dataContainer");
        if (Strings.isNullOrEmpty(containerId)) {
            throw new GuiDevelopmentException("DynamicAttributesPanel element doesn't have 'dataContainer' attribute",
                    context, "DynamicAttributesPanel ID", element.attributeValue("id"));
        }
        FrameOwner frameOwner = getComponentContext().getFrame().getFrameOwner();
        ScreenData screenData = UiControllerUtils.getScreenData(frameOwner);
        InstanceContainer container = screenData.getContainer(containerId);
        //noinspection unchecked
        resultComponent.setInstanceContainer(container);
    }

    protected void loadColumnsCount(ProblemDynamicAttributesPanel resultComponent, Element element) {
        resultComponent.setColumnsCount(getIntegerAttribute("cols", element));
    }

    protected void loadRowsCount(ProblemDynamicAttributesPanel resultComponent, Element element) {
        resultComponent.setRowsCount(getIntegerAttribute("rows", element));
    }

    protected void loadFieldWidth(ProblemDynamicAttributesPanel resultComponent, Element element) {
        String fieldWidth = element.attributeValue("fieldWidth");
        if (!Strings.isNullOrEmpty(fieldWidth)) {
            resultComponent.setFieldWidth(fieldWidth);
        }
    }

    protected void loadFieldCaptionWidth(ProblemDynamicAttributesPanel resultComponent, Element element) {
        String fieldWidth = element.attributeValue("fieldCaptionWidth");
        if (!Strings.isNullOrEmpty(fieldWidth)) {
            resultComponent.setFieldCaptionWidth(fieldWidth);
        }
    }

    protected Integer getIntegerAttribute(String attributeName, Element element) {
        String columnsCountStr = element.attributeValue(attributeName);
        if (!Strings.isNullOrEmpty(columnsCountStr)) {
            return Integer.parseInt(columnsCountStr);
        }
        return null;
    }
}

