package ru.mkb.corp.monmod.web.screens.covenant;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.gui.components.TabSheet;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.security.global.UserSession;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.service.user.UserService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@UiController("mm_CommonCovenantFragment")
@UiDescriptor("common-covenant-fragment.xml")
public class CommonCovenantFragment extends ScreenFragment {

    @Inject
    private TabSheet covenantTabSheet;
    @Inject
    private CollectionLoader<CovenantMonCard> covenantsDl;
    @Inject
    private DataManager dataManager;
    @Inject
    private UserSession userSession;
    @Inject
    private InstanceContainer<MonitoringCard> monitoringCardDc;
    @Inject
    private UserService userService;


    @Subscribe(target = Target.PARENT_CONTROLLER)
    public void onAfterShow(Screen.AfterShowEvent event) {
        covenantsDl.setParameter("ids", getCovenantIds(monitoringCardDc.getItem()));
        covenantsDl.setParameter("financialConditionType", "Финансовый");
        covenantsDl.load();
        covenantTabSheet.addSelectedTabChangeListener(selectedTabChangeEvent -> {
            covenantsDl.setParameter("ids", getCovenantIds(monitoringCardDc.getItem()));
            if ("finCovenantTab".equals(selectedTabChangeEvent.getSelectedTab().getName())) {
                covenantsDl.setParameter("financialConditionType", "Финансовый");
            } else if ("nonFinCovenantTab".equals(selectedTabChangeEvent.getSelectedTab().getName())) {
                covenantsDl.setParameter("financialConditionType", "Нефинансовый");
            }
            covenantsDl.load();
        });
    }

    public List<UUID> getCovenantIds(MonitoringCard monitoringCard) {
        List<UUID> ids = new ArrayList<>();
        List<CovenantMonCard> covenants = dataManager.load(CovenantMonCard.class)
                .query("e.monCard =:monCard")
                .parameter("monCard", monitoringCard)
                .view("covenantMonCard-view")
                .list();
        if (userService.isMonManager(userSession.getCurrentOrSubstitutedUser())) {
            covenants.forEach(covenantMonCard -> ids.add(covenantMonCard.getId()));
        } else {
            for (RespEmployee respEmployee : monitoringCard.getRespEmp()) {
                if (!covenants.isEmpty() && (respEmployee.getUser().equals(userSession.getCurrentOrSubstitutedUser())
                        || respEmployee.getSubdivision().getLeader().equals(userSession.getCurrentOrSubstitutedUser())))
                    covenants.stream()
                            .filter(covenant -> covenant.getCovenant().getControlSubdivision().getSubdivision().equals(respEmployee.getSubdivision().getSubdivision())
                                    || covenant.getCovenant().getInfoSubdivision().getSubdivision().equals(respEmployee.getSubdivision().getSubdivision())
                                    || covenant.getCovenant().getExpertSubdivision().getSubdivision().equals(respEmployee.getSubdivision().getSubdivision()))
                            .forEach(covenant -> ids.add(covenant.getId()));
            }
        }
        return ids;
    }



}