package ru.mkb.corp.monmod.web.screens.covenant;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.GroupTable;
import com.haulmont.cuba.gui.components.Table;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.screen.Install;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.security.global.UserSession;
import ru.mkb.corp.monmod.entity.covenant.CovenantMonCard;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.CovenantEvent;
import ru.mkb.corp.monmod.entity.event.FinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.event.NonFinancialCovenantEvent;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;

import javax.inject.Inject;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@UiController("mm_CovenantFragment")
@UiDescriptor("covenant-fragment.xml")
public class CovenantFragment extends ScreenFragment {

    @Inject
    private CollectionContainer<CovenantEvent> covenantEventDc;
    @Inject
    private DataManager dataManager;
    @Inject
    private GroupTable<CovenantEvent> eventTable;
    @Inject
    private ScreenBuilders screenBuilders;
    @Inject
    private UserSession userSession;

    @Subscribe("covenantTable")
    public void onCovenantTableSelection(Table.SelectionEvent<CovenantMonCard> event) {
        if (event.getSource().getSingleSelected() != null) {
        covenantEventDc.setItems(dataManager.load(CovenantEvent.class)
                .query("e.monitoringCard =:monitoringCard and e.covenant.covenantMonCard =:covenantMonCard")
                .parameter("monitoringCard", event.getSource().getSingleSelected().getMonCard())
                .parameter("covenantMonCard", event.getSource().getSingleSelected())
                .view("covenantEvent-view")
                .list());
        }
        else covenantEventDc.setItems(null);
    }

    @Subscribe("eventTable.view")
    public void onEventTableView(Action.ActionPerformedEvent event) {
        CovenantEvent covenantEvent = eventTable.getSingleSelected();
        if (covenantEvent instanceof FinancialCovenantEvent) {
            screenBuilders.editor(FinancialCovenantEvent.class, this)
                    .withOpenMode(OpenMode.DIALOG)
                    .editEntity((FinancialCovenantEvent) covenantEvent)
                    .show();
        }
        if (covenantEvent instanceof NonFinancialCovenantEvent) {
            screenBuilders.editor(NonFinancialCovenantEvent.class, this)
                    .withOpenMode(OpenMode.DIALOG)
                    .editEntity((NonFinancialCovenantEvent) covenantEvent)
                    .show();
        }
    }

    @Install(to = "eventTable", subject = "styleProvider")
    private String eventTableStyleProvider(CovenantEvent entity, String property) {
        if (property == null) {
            if (entity.getCompleted()) {
                return "event-grey";
            } else if (entity.getPlannedEndDate().isBefore(LocalDate.now())) {
                return "event-red";
            } else return  "event-blue";
        }
        return null;
    }



}