package ru.mkb.corp.monmod.web.screens.eventtype;

import com.haulmont.addon.bproc.entity.ProcessDefinitionData;
import com.haulmont.addon.bproc.query.ProcessDefinitionDataQuery;
import com.haulmont.addon.bproc.service.BprocRepositoryService;
import com.haulmont.addon.bproc.web.services.TenantUiHelper;
import com.haulmont.cuba.core.global.LoadContext;
import com.haulmont.cuba.gui.Fragments;
import com.haulmont.cuba.gui.components.Component;
import com.haulmont.cuba.gui.components.HasValue;
import com.haulmont.cuba.gui.components.LookupField;
import com.haulmont.cuba.gui.components.ScrollBoxLayout;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.*;
import com.haulmont.cuba.security.global.UserSession;
import ru.mkb.corp.monmod.entity.event.EventType;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEventMonitoringFactor;
import ru.mkb.corp.monmod.entity.event.settings.SettingsCD;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFK;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFP;
import ru.mkb.corp.monmod.entity.event.settings.SettingsNFK;
import ru.mkb.corp.monmod.web.screens.problemfactorevent.EventProblemFragment;
import ru.mkb.corp.monmod.web.screens.problemfactorevent.ObserveProblemFactor;
import ru.mkb.corp.monmod.web.screens.settingscd.SettingsCDFragment;
import ru.mkb.corp.monmod.web.screens.settingsfk.SettingsfkFragment;
import ru.mkb.corp.monmod.web.screens.settingsfp.SettingsFPFragment;
import ru.mkb.corp.monmod.web.screens.settingsnfk.SettingsnfkFragment;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@UiController("mm_EventType.edit")
@UiDescriptor("event-type-edit.xml")
@EditedEntityContainer("eventTypeDc")
@LoadDataBeforeShow
public class EventTypeEdit extends StandardEditor<EventType> {
    @Inject
    private ScrollBoxLayout settingsFragments;
    @Inject
    private Fragments fragments;
    @Inject
    private LookupField<ProcessDefinitionData> processField;
    @Inject
    private BprocRepositoryService bprocRepositoryService;
    @Inject
    private TenantUiHelper tenantUiHelper;
    @Inject
    private InstanceContainer<EventType> eventTypeDc;
    @Inject
    private CollectionContainer<ProcessDefinitionData> processDc;


    @Install(to = "processDl", target = Target.DATA_LOADER)
    private List<ProcessDefinitionData> processDlLoadDelegate(LoadContext<ProcessDefinitionData> loadContext) {
        ProcessDefinitionDataQuery query = bprocRepositoryService.createProcessDefinitionDataQuery();
        if (tenantUiHelper.isUserTenantInQuery()) {
            query.processDefinitionTenantId(tenantUiHelper.getCurrentUserTenantId());
        }
        return query
                .latestVersion()
                .active()
                .orderByProcessDefinitionName()
                .asc()
                .list();
    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        if (getEditedEntity().getSettings() instanceof SettingsFK) {
            SettingsFK settingsFK = (SettingsFK) getEditedEntity().getSettings();
            settingsFragments.add(createFKFragment(settingsFK));
        } else if (getEditedEntity().getSettings() instanceof SettingsNFK) {
            SettingsNFK settingsNFK = (SettingsNFK) getEditedEntity().getSettings();
            settingsFragments.add(createNFKFragment(settingsNFK));
        } else if (getEditedEntity().getSettings() instanceof SettingsFP) {
            SettingsFP settingsFP = (SettingsFP) getEditedEntity().getSettings();
            settingsFragments.add(createFPFragment(settingsFP));
        } else if (getEditedEntity().getSettings() instanceof SettingsCD) {
            SettingsCD settingsCD = (SettingsCD) getEditedEntity().getSettings();
            settingsFragments.add(createCDFragment(settingsCD));
        }

        Optional.ofNullable(getEditedEntity().getSettings().getProcess())
                .ifPresent(s -> {
                    processDc.getMutableItems().stream()
                            .filter(processDefinitionData -> processDefinitionData.getKey().equals(s))
                            .findFirst()
                            .ifPresent(processDefinitionData -> {
                                processField.setValue(processDefinitionData);
                            });
                });
    }

    private Component createFKFragment(SettingsFK settingsFK) {
        SettingsfkFragment settingsfkFragment = fragments.create(this, SettingsfkFragment.class);
        settingsfkFragment.setInstance(getEditedEntity());
        return settingsfkFragment.getFragment();
    }

    private Component createNFKFragment(SettingsNFK settingsNFK) {
        SettingsnfkFragment settingsnfkFragment = fragments.create(this, SettingsnfkFragment.class);
        settingsnfkFragment.setInstance(getEditedEntity());
        return settingsnfkFragment.getFragment();
    }

    private Component createFPFragment(SettingsFP settingsFP) {
        SettingsFPFragment settingsFPFragment = fragments.create(this, SettingsFPFragment.class);
        settingsFPFragment.setInstance(getEditedEntity());
        return settingsFPFragment.getFragment();
    }

    private Component createCDFragment(SettingsCD settingsCD) {
        SettingsCDFragment settingsCDFragment = fragments.create(this, SettingsCDFragment.class);
        settingsCDFragment.setInstance(getEditedEntity());
        return settingsCDFragment.getFragment();
    }


    @Subscribe("processField")
    public void onProcessFieldValueChange(HasValue.ValueChangeEvent<ProcessDefinitionData> event) {
        getEditedEntity().getSettings().setProcess(Optional.ofNullable(event.getValue()).map(ProcessDefinitionData::getKey).orElse(null));
    }

    @Install(to = "processField", subject = "optionCaptionProvider")
    private String processFieldOptionCaptionProvider(ProcessDefinitionData processDefinitionData) {
        return processDefinitionData.getName();
    }
}