package ru.mkb.corp.monmod.web.screens.factor;

import com.haulmont.cuba.core.entity.CategoryAttributeValue;
import com.haulmont.cuba.core.global.Scripting;
import com.haulmont.cuba.gui.components.CheckBox;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.model.InstancePropertyContainer;
import com.haulmont.cuba.gui.screen.EditedEntityContainer;
import com.haulmont.cuba.gui.screen.LoadDataBeforeShow;
import com.haulmont.cuba.gui.screen.StandardEditor;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import groovy.lang.Binding;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttributeHolder;
import ru.mkb.corp.monmod.entity.problemfactor.Factor;
import ru.mkb.corp.monmod.entity.problemfactorattribute.ProblemFactorAttribute;

import javax.inject.Inject;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@UiController("mm_Factor.edit")
@UiDescriptor("factor-edit.xml")
@EditedEntityContainer("factorDc")
@LoadDataBeforeShow
public class FactorEdit extends StandardEditor<Factor> {

    @Inject
    private InstancePropertyContainer<ProblemAttributeHolder> problemAttributeHolderDc;

    @Inject
    Scripting scripting;

    @Inject
    private CheckBox isIdentifiedField;

    @Inject
    private InstanceContainer<Factor> factorDc;


    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        detectedProblem();
    }



    @Subscribe(id = "problemAttributeHolderDc", target = Target.DATA_CONTAINER)
    public void onProblemAttributeHolderDcItemPropertyChange(InstanceContainer.ItemPropertyChangeEvent<ProblemAttributeHolder> event) {
        detectedProblem();
    }

    public void detectedProblem() {
        if (factorDc.getItem().getProblemFactor() != null) {
            final Map<String, CategoryAttributeValue> dynamicAttributes = problemAttributeHolderDc.getItem().getDynamicAttributes();
            String script = factorDc.getItem().getProblemFactor().getScript();
            Binding binding = new Binding();
            List<ProblemFactorAttribute> pfa = problemAttributeHolderDc.getItem().getFactor().getAttributes();
            for (ProblemFactorAttribute pf : pfa) {
                binding.setVariable(pf.getAttribute().getCode(),
                        Optional.ofNullable(dynamicAttributes.get(pf.getAttribute().getCode()))
                                .map(CategoryAttributeValue::getValue)
                                .orElse(null)
                );
            }
            Boolean b = scripting.evaluateGroovy(script, binding);
            isIdentifiedField.setValue(b);
        }
    }

}