package ru.mkb.corp.monmod.web.screens.file;

import com.haulmont.cuba.core.entity.FileDescriptor;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.FileStorageException;
import com.haulmont.cuba.core.global.TimeSource;
import com.haulmont.cuba.core.global.UserSessionSource;
import com.haulmont.cuba.gui.Dialogs;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.DataGrid;
import com.haulmont.cuba.gui.components.FileMultiUploadField;
import com.haulmont.cuba.gui.export.ExportDisplay;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.Install;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.gui.upload.FileUploadingAPI;
import ru.mkb.corp.monmod.entity.directories.File;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;

import javax.inject.Inject;

@UiController("mm_FileFragment")
@UiDescriptor("file-fragment.xml")
public class FileFragment extends ScreenFragment {

    @Inject
    private FileUploadingAPI fileUploadingAPI;
    @Inject
    private CollectionContainer<File> fileDc;
    @Inject
    private DataGrid<File> filesGrid;

    @Inject
    private ExportDisplay exportDisplay;

    @Inject
    private Dialogs dialogs;

    @Inject
    protected ScreenBuilders screenBuilders;

    @Inject
    private DataManager dataManager;

    @Inject
    private UserSessionSource userSessionSource;

    @Inject
    private TimeSource timeSource;
    @Inject
    private InstanceContainer<MonitoringCard> monitoringCardDc;

    @Subscribe(target = Target.PARENT_CONTROLLER)
    public void onInit(Screen.InitEvent event) {
        initRenderers();
    }

    protected void initRenderers() {
        DataGrid.ClickableTextRenderer<FileDescriptor> fileName = filesGrid.createRenderer(DataGrid.ClickableTextRenderer.class);
        fileName.setRendererClickListener(clickEvent -> {
            FileDescriptor item = clickEvent.getItem();
            exportDisplay.show(item);
        });
        fileName.setNullRepresentation("Скачать файл");
        filesGrid.getColumnNN("download").setRenderer(fileName);
    }

    @Subscribe("filesGrid.downloadAction")
    public void onFilesGridDownload(Action.ActionPerformedEvent event) {
        filesGrid.getSelected().forEach(item -> {
            exportDisplay.show(item.getFileDescriptor());
        });
    }

    @Install(to = "filesGrid.fileUpload", subject = "initializer")
    private void filesGridFileUploadInitializer(File file) {
        file.setMonitoringCard(this.monitoringCardDc.getItem());
        file.setLastModDate(timeSource.currentTimestamp());
        file.setAuthor(userSessionSource.getUserSession().getCurrentOrSubstitutedUser());
    }

}