package ru.mkb.corp.monmod.web.screens.monitoringcard;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.gui.UiComponents;
import com.haulmont.cuba.gui.components.CheckBox;
import com.haulmont.cuba.gui.components.DataGrid;
import com.haulmont.cuba.gui.components.DateField;
import com.haulmont.cuba.gui.components.Field;
import com.haulmont.cuba.gui.components.GroupBoxLayout;
import com.haulmont.cuba.gui.components.HasValue;
import com.haulmont.cuba.gui.components.LookupField;
import com.haulmont.cuba.gui.components.PickerField;
import com.haulmont.cuba.gui.components.TextField;
import com.haulmont.cuba.gui.components.data.ValueSource;
import com.haulmont.cuba.gui.model.CollectionPropertyContainer;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.Install;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.security.entity.Group;
import com.haulmont.cuba.security.entity.User;
import com.haulmont.cuba.web.gui.components.WebTextField;
import org.springframework.util.StringUtils;
import ru.mkb.corp.monmod.entity.counterparty.Client;
import ru.mkb.corp.monmod.entity.counterparty.Counterparty;
import ru.mkb.corp.monmod.entity.counterparty.Holding;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.limit.ClientType;
import ru.mkb.corp.monmod.entity.limit.LimitGraph;
import ru.mkb.corp.monmod.entity.limit.RelationWithLimit;
import ru.mkb.corp.monmod.entity.monitoring.ClientClass;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.monitoring.PvLevel;

import javax.inject.Inject;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@UiController("mm_CommonFragment")
@UiDescriptor("common-fragment.xml")
public class CommonFragment extends ScreenFragment {

    @Inject
    protected LookupField<PvLevel> pvLevelField;
    @Inject
    protected LookupField<ClientClass> clientClassField;
    @Inject
    InstanceContainer<MonitoringCard> monitoringCardDc;
    @Inject
    CollectionPropertyContainer<RespEmployee> respEmpDc;
    @Inject
    private CheckBox havePvCheckbox;
    @Inject
    private PickerField<Counterparty> clientNameField;
    @Inject
    private InstanceContainer<Client> clientDc;
    @Inject
    private GroupBoxLayout clientGroupBox;
    @Inject
    private TextField<String> clientINNField;
    @Inject
    private TextField<Holding> holdingField;
    @Inject
    private DataManager dataManager;
    @Inject
    private InstanceContainer<Holding> holdingDc;
    @Inject
    private UiComponents uiComponents;
    @Inject
    private TextField<String> holdingLimitField;
    @Inject
    private TextField<String> clientLimitField;
    @Inject
    private DateField<Date> startDateLimitField;
    @Inject
    private DateField<Date> endDateLimitField;
    @Inject
    private TextField<String> currentLimitField;
    @Inject
    private CollectionPropertyContainer<RelationWithLimit> relationDc;

    @Subscribe(target = Target.PARENT_CONTROLLER)
    protected void onInit(Screen.InitEvent event) {
        havePvCheckbox.addValueChangeListener(valueChangeEvent -> {
            if (Boolean.TRUE.equals(valueChangeEvent.getValue())) {
                pvLevelField.setEditable(true);
            } else {
                pvLevelField.setEditable(false);
                pvLevelField.setValue(PvLevel.NONE);
            }
        });
    }

    @Subscribe(target = Target.PARENT_CONTROLLER)
    public void onAfterShow(Screen.AfterShowEvent event) {
        updateVisibility();
        setDataLimit();
    }

    public List<User> usersByRole(String roleName, Group group) {

        return dataManager.load(User.class)
                .query("select distinct ur.user from sec$UserRole ur where ur.role.name =:roleName and ur.user.group =:group")
                .parameter("group", group)
                .parameter("roleName", roleName)
                .list();
    }


    @Subscribe("pvLevelField")
    public void onPvLevelFieldValueChange(HasValue.ValueChangeEvent<PvLevel> event) {

        clientClassField.setValue(pvLevelField.getValue().getClientClass());

    }

    private void updateVisibility() {
        Counterparty cp = clientNameField.getValue();
        boolean hasClient = !StringUtils.isEmpty(cp);
        clientGroupBox.getComponents()
                .stream()
                .filter(component -> WebTextField.class.isAssignableFrom(component.getClass()))
                .forEach(component -> component.setVisible(hasClient));
        clientDc.setItem(null);
        holdingDc.setItem(null);
        if (hasClient) {
            if (cp instanceof Holding) {
                holdingDc.setItem(dataManager.reload((Holding) cp, holdingDc.getView()));
                clientDc.setItem(null);
                clientINNField.setVisible(false);
                holdingField.setVisible(false);
            } else if (cp instanceof Client) {
                clientDc.setItem(dataManager.reload((Client) cp, clientDc.getView()));
                holdingDc.setItem(null);
                holdingField.setVisible(!StringUtils.isEmpty(holdingField.getRawValue()));
                clientINNField.setVisible(true);
            } else {
                holdingField.setVisible(false);
                clientINNField.setVisible(false);
            }
        }
    }

    @Subscribe("clientNameField")
    public void onClientNameFieldValueChange(HasValue.ValueChangeEvent<Counterparty> event) {
        updateVisibility();
    }

    @Install(to = "respEmpDataGrid.user", subject = "editFieldGenerator")
    private Field<User> respEmpDataGridUserEditFieldGenerator(DataGrid.EditorFieldGenerationContext<RespEmployee> editorFieldGenerationContext) {
        LookupField<User> lookupField = uiComponents.create(LookupField.NAME);
        lookupField.setValueSource((ValueSource<User>) editorFieldGenerationContext.getValueSourceProvider().getValueSource("user"));
        lookupField.setOptionsList(usersByRole("manager", editorFieldGenerationContext.getItem().getSubdivision().getSubdivision()));
        respEmpDc.addItemPropertyChangeListener(even -> {
            if (even.getItem().getSubdivision().getName().equals("ОМКП")) {
                monitoringCardDc.getItem().setMonManager(respEmpDc.getItem().getUser());
            }
        });
        return lookupField;

    }

    private void setDataLimit() {
        LimitGraph limitGraph = relationDc.getMutableItems()
                .stream()
                .flatMap(limit -> limit.getLimit().getLimitGraphs()
                        .stream()
                        .filter(lg -> lg.getEndDate().isAfter(LocalDate.now()) || lg.getEndDate().isEqual(lastDateLimit())))
                .findFirst()
                .orElse(null);
        if (limitGraph != null) {
            startDateLimitField.setValue(Date.from(limitGraph.getStartDate().atStartOfDay(ZoneId.systemDefault()).toInstant()));
            endDateLimitField.setValue(Date.from(limitGraph.getEndDate().atStartOfDay(ZoneId.systemDefault()).toInstant()));
            if (clientDc.getItemOrNull() != null) {
                currentLimitField.setValue("Лимит на " + clientDc.getItem().getName());
                if (clientDc.getItem().getHolding() != null)
                    holdingLimitField.setValue(limitGraph.getAmount().toString());
                amountClient();
            } else {
                currentLimitField.setValue("Лимит на " + holdingDc.getItem().getName());
                holdingLimitField.setValue(limitGraph.getAmount().toString());
                clientLimitField.setValue(limitGraph.getAmount().toString());
            }
        }
    }

    private LocalDate lastDateLimit() {
        return relationDc.getMutableItems()
                .stream()
                .flatMap(limit -> limit.getLimit().getLimitGraphs()
                        .stream()
                        .map(limitGraph -> limitGraph.getEndDate()))
                .max(Comparator.comparing(LocalDate::toEpochDay))
                .orElse(null);
    }


    private void amountClient() {
        BigDecimal amount = new BigDecimal(0);
        RelationWithLimit relationWithLimits = relationDc.getMutableItems()
                .stream()
                .flatMap(relation -> relation.getClientRoles()
                        .stream()
                        .filter(roleClient -> roleClient.getClientType().equals(ClientType.BORROWER)))
                .map(roleClient -> roleClient.getRelation())
                .findFirst()
                .orElse(null);

        if (relationWithLimits != null) {
            List<LimitGraph> limitGraphs = relationWithLimits.getLimit().getSublimits()
                    .stream()
                    .flatMap(sublimit -> sublimit.getPodlimits()
                            .stream())
                    .flatMap(podlimit -> podlimit.getPodlimitGraphs()
                            .stream())
                    .collect(Collectors.toList());
            for (LimitGraph limitGraph : limitGraphs) {
                amount = amount.add(limitGraph.getAmount());
            }
            clientLimitField.setValue(amount.toString());
        } else clientLimitField.setValue(amount.toString());
    }

}