package ru.mkb.corp.monmod.web.screens.monitoringcard;

import com.haulmont.chile.core.model.MetaClass;
import com.haulmont.chile.core.model.Session;
import com.haulmont.cuba.core.app.EntitySnapshotService;
import com.haulmont.cuba.core.entity.Entity;
import com.haulmont.cuba.core.entity.EntitySnapshot;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.Metadata;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.CheckBox;
import com.haulmont.cuba.gui.components.GroupTable;
import com.haulmont.cuba.gui.components.HasValue;
import com.haulmont.cuba.gui.components.LookupField;
import com.haulmont.cuba.gui.components.Table;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.*;
import com.haulmont.cuba.security.entity.User;
import com.haulmont.cuba.security.global.UserSession;
import org.slf4j.Logger;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.service.event.EventService;
import ru.mkb.corp.monmod.service.user.UserService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

@UiController("mm_MonitoringCard.browse")
@UiDescriptor("monitoring-card-browse.xml")
@LookupComponent("monitoringCardsTable")
public class MonitoringCardBrowse extends StandardLookup<MonitoringCard> {
    @Inject
    EventService eventService;
    @Inject
    private GroupTable<MonitoringCard> monitoringCardsTable;
    @Inject
    private CollectionLoader<CommonAttributesEvent> commonAttributesEventDl;
    @Inject
    private CollectionLoader<MonitoringCard> monitoringCardsDl;
    @Inject
    private UserSession userSession;
    @Inject
    private DataManager dataManager;
    @Inject
    private UserService userService;
    @Inject
    private CheckBox completed;


    @Subscribe("generateEvent")
    public void onGenerateEventClick(Button.ClickEvent event) {
        eventService.generateEvent(monitoringCardsTable.getSingleSelected());
    }

    @Subscribe("monitoringCardsTable")
    public void onMonitoringCardsTableSelection(Table.SelectionEvent<MonitoringCard> event) {
        commonAttributesEventDl.setParameter("ids", getEventIds(event.getSource().getSingleSelected()));
        if (completed.isChecked()) {
            commonAttributesEventDl.setParameter("completed", true);
        } else {
            commonAttributesEventDl.setParameter("completed", false);
        }
        commonAttributesEventDl.load();
    }

    @Subscribe("completed")
    public void onCompletedValueChange(HasValue.ValueChangeEvent<Boolean> event) {
        if (event.getValue()) {
            commonAttributesEventDl.setParameter("completed", null);
        } else {
            commonAttributesEventDl.setParameter("completed", false);
        }
        if (monitoringCardsTable.getSingleSelected() != null) {
            commonAttributesEventDl.load();
        }
    }

    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {
        if (!userService.isLeader(userSession.getCurrentOrSubstitutedUser()) &&
                !userService.isMonManager(userSession.getCurrentOrSubstitutedUser())) {
            monitoringCardsDl.setParameter("ids", getMonCardIds());
        }
        monitoringCardsDl.load();
        commonAttributesEventDl.setParameter("completed", false);
    }

    public List<UUID> getMonCardIds() {
        List<UUID> ids = new ArrayList<>();
        dataManager.load(RespEmployee.class)
                .query("e.user =:user")
                .parameter("user", userSession.getCurrentOrSubstitutedUser() )
                .view("respEmployeeWithMonCard-view")
                .list()
                .forEach(respEmployee -> ids.add(respEmployee.getMonCard().getId()));
        return ids;
    }

    public List<UUID> getEventIds(MonitoringCard monitoringCard) {
        List<UUID> ids = new ArrayList<>();
        List<CommonAttributesEvent> events = dataManager.load(CommonAttributesEvent.class)
                .query("e.monitoringCard =:monitoringCard")
                .parameter("monitoringCard", monitoringCard)
                .view("commonAttributesEvent-view")
                .list();
        if (userService.isMonManager(userSession.getCurrentOrSubstitutedUser())) {
            events.forEach(event -> ids.add(event.getId()));
        } else {
        for (RespEmployee respEmployee : monitoringCard.getRespEmp()) {
            if (!events.isEmpty() && (respEmployee.getUser().equals(userSession.getCurrentOrSubstitutedUser())
                    || respEmployee.getSubdivision().getLeader().equals(userSession.getCurrentOrSubstitutedUser())))
                events.stream()
                        .flatMap(event -> event.getExecutors().stream())
                        .filter(executor -> executor.getGroup().equals(respEmployee.getSubdivision().getSubdivision()))
                        .forEach(executor -> ids.add(executor.getEvent().getId()));
        }
        }
        return ids;
    }

    @Subscribe(id = "monitoringCardsDc", target = Target.DATA_CONTAINER)
    public void onMonitoringCardsDcItemPropertyChange(InstanceContainer.ItemPropertyChangeEvent<MonitoringCard> event) {
        monitoringCardsDl.setParameter("ids", getMonCardIds());
        monitoringCardsDl.load();
    }
}