package ru.mkb.corp.monmod.web.screens.monitoringcard;

import com.haulmont.cuba.core.app.EntitySnapshotService;
import com.haulmont.cuba.core.entity.EntitySnapshot;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.PersistenceHelper;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.HBoxLayout;
import com.haulmont.cuba.gui.model.CollectionPropertyContainer;
import com.haulmont.cuba.gui.model.DataContext;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.*;
import ru.mkb.corp.monmod.entity.directories.Subdivision;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCardVersion;

import javax.inject.Inject;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.List;
import java.util.Locale;

@UiController("mm_MonitoringCard.edit")
@UiDescriptor("monitoring-card-edit.xml")
@EditedEntityContainer("monitoringCardDc")
@LoadDataBeforeShow
public class MonitoringCardEdit extends StandardEditor<MonitoringCard> {

    @Inject
    DataManager dataManager;
    @Inject
    CollectionPropertyContainer<RespEmployee> respEmpDc;
    @Inject
    protected InstanceContainer<MonitoringCard> monitoringCardDc;
    @Inject
    DataContext dataContext;
    @Inject
    private EntitySnapshotService entitySnapshotService;
    @Inject
    private Button commitAndCloseBtn;
    @Inject
    private Button closeBtn;
    @Inject
    private Button closeVersionBtn;
    @Inject
    private HBoxLayout commonInformationBox;

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {

        if (PersistenceHelper.isNew(getEditedEntity())) {
            getWindow().setCaption(getEditedEntity().getNumber());
            List<Subdivision> subdivisions = dataManager.load(Subdivision.class)
                    .query("select sd from mm_Subdivision sd")
                    .view("subdivision-view")
                    .list();
            for (Subdivision subdivision : subdivisions) {
                RespEmployee re = dataContext.create(RespEmployee.class);
                re.setSubdivision(subdivision);
                re.setName(subdivision.getName());
                re.setMonCard(getEditedEntity());
                re.setUser(subdivision.getManager());
                getEditedEntity().getRespEmp().add(re);
                if (subdivision.getName().equals("ОМКП")) {
                    getEditedEntity().setMonManager(subdivision.getManager());
                }
            }
            respEmpDc.setItems(getEditedEntity().getRespEmp());
        } else {
            getWindow().setCaption(getEditedEntity().getNumber() + " на " + monitoringCardDc.getItem().getClient().getName());
        }
        showVersionField();
    }

    @Subscribe
    public void onAfterCommitChanges(AfterCommitChangesEvent event) {
       createVersion();
    }

    @Subscribe("closeVersionBtn")
    public void onCloseVersionBtnClick(Button.ClickEvent event) {
        closeWithDiscard();
    }


    public void showVersionField() {
        if (isReadOnly()) {
            commitAndCloseBtn.setVisible(false);
            closeBtn.setVisible(false);
            commonInformationBox.setVisible(false);
        } else {
            closeVersionBtn.setVisible(false);
        }
    }

    public void createVersion() {
        EntitySnapshot snapshot = entitySnapshotService.createSnapshot(monitoringCardDc.getItem(), monitoringCardDc.getView());
        MonitoringCardVersion monitoringCardVersion = dataManager.create(MonitoringCardVersion.class);
        monitoringCardVersion.setMonCard(getEditedEntity());
        monitoringCardVersion.setSnapshot(snapshot);
        LocalDateTime createDate = snapshot.getSnapshotDate().toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        monitoringCardVersion.setShortName("Версия №" + getEditedEntity().getVersion().toString() + " от " +
                new SimpleDateFormat("dd MMMM yyyy HH:mm:ss", Locale.getDefault()).format(getEditedEntity().getUpdateTs()));
        monitoringCardVersion.setCreateDate(createDate.toLocalDate());
        dataManager.commit(monitoringCardVersion);
    }

    public void setItem(MonitoringCard monitoringCard) {
        monitoringCardDc.setItem(monitoringCard);
    }
}