package ru.mkb.corp.monmod.web.screens.monitoringcard;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.Screens;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.DataGrid;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.security.global.UserSession;
import ru.mkb.corp.monmod.entity.directories.ProblemFactor;
import ru.mkb.corp.monmod.entity.directories.RespEmployee;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.problemfactor.Factor;
import ru.mkb.corp.monmod.service.user.UserService;
import ru.mkb.corp.monmod.web.screens.problemfactor.ProblemFactorBrowse;
import ru.mkb.corp.monmod.web.screens.problemfactor.ProblemFactorEdit;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@UiController("mm_ProblemFragment")
@UiDescriptor("problem-fragment.xml")
public class ProblemFragment extends ScreenFragment {

    @Inject
    private CollectionLoader<Factor> factorsDl;
    @Inject
    private InstanceContainer<MonitoringCard> monitoringCardDc;
    @Inject
    private DataManager dataManager;
    @Inject
    private UserSession userSession;
    @Inject
    private UserService userService;


    @Subscribe(target = Target.PARENT_CONTROLLER)
    public void onAfterShow(Screen.AfterShowEvent event) {
        factorsDl.setParameter("ids", getProblemFactorIds(monitoringCardDc.getItem()));
        factorsDl.load();
    }



    public List<UUID> getProblemFactorIds(MonitoringCard monitoringCard) {
        List<UUID> ids = new ArrayList<>();
        List<Factor> factors = dataManager.load(Factor.class)
                .query("e.monCard =:monCard")
                .parameter("monCard", monitoringCard)
                .view("factor-view")
                .list();
        if (userService.isMonManager(userSession.getCurrentOrSubstitutedUser())) {
            factors.forEach(factor -> ids.add(factor.getId()));
        } else {
            for (RespEmployee respEmployee : monitoringCard.getRespEmp()) {
                if (!factors.isEmpty() && (respEmployee.getUser().equals(userSession.getCurrentOrSubstitutedUser())
                        || respEmployee.getSubdivision().getLeader().equals(userSession.getCurrentOrSubstitutedUser())))
                    factors.stream()
                            .filter(factor -> factor.getSubdivision().getSubdivision().equals(respEmployee.getSubdivision().getSubdivision())
                                    || respEmployee.getName().equals("ОМКП"))
                            .forEach(factor -> ids.add(factor.getId()));
            }
        }
        return ids;
    }

}