package ru.mkb.corp.monmod.web.screens.monitoringcard;

import com.haulmont.cuba.core.app.EntitySnapshotService;
import com.haulmont.cuba.core.entity.EntitySnapshot;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.DateField;
import com.haulmont.cuba.gui.components.GroupTable;
import com.haulmont.cuba.gui.components.HasValue;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCard;
import ru.mkb.corp.monmod.entity.monitoring.MonitoringCardVersion;

import javax.inject.Inject;
import java.time.ZoneId;
import java.util.Date;

@UiController("mm_VersionFragment")
@UiDescriptor("version-fragment.xml")
public class VersionFragment extends ScreenFragment {
    @Inject
    private InstanceContainer<MonitoringCard> monitoringCardDc;
    @Inject
    private CollectionLoader<MonitoringCardVersion> versionsDl;
    @Inject
    private ScreenBuilders screenBuilders;
    @Inject
    private GroupTable<MonitoringCardVersion> versionTable;
    @Inject
    private EntitySnapshotService entitySnapshotService;
    @Inject
    private DataManager dataManager;

    @Subscribe(target = Target.PARENT_CONTROLLER)
    public void onAfterShow(Screen.AfterShowEvent event) {
        versionsDl.setParameter("monCard", monitoringCardDc.getItem());
        versionsDl.load();
    }

    @Subscribe("versionDateField")
    public void onVersionDateFieldValueChange(HasValue.ValueChangeEvent<Date> event) {
        if (event.getValue() != null) {
        versionsDl.setParameter("createDate", event.getValue().toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
        } else {
            versionsDl.setParameter("createDate", null);
        }
        versionsDl.load();
    }

    @Subscribe("versionTable.view")
    public void onVersionTableView(Action.ActionPerformedEvent event) {
        EntitySnapshot entitySnapshot = versionTable.getSingleSelected().getSnapshot();
        MonitoringCard monitoringCard = (MonitoringCard) entitySnapshotService.extractEntity(entitySnapshot);
        MonitoringCardEdit monitoringCardEdit = screenBuilders.editor(MonitoringCard.class, this)
                .withScreenClass(MonitoringCardEdit.class)
                .editEntity(versionTable.getSingleSelected().getMonCard())
                .withOpenMode(OpenMode.NEW_TAB)
                .build();
        monitoringCardEdit.setReadOnly(true);
        monitoringCardEdit.show();
        monitoringCardEdit.setItem(monitoringCard);
    }

    @Subscribe("approvedCheckBox")
    public void onApprovedCheckBoxValueChange(HasValue.ValueChangeEvent<Boolean> event) {
        if (event.getValue()) {
            versionsDl.setParameter("approved", true);
        } else {
            versionsDl.setParameter("approved", null);
        }
        versionsDl.load();
    }

    @Subscribe("approveBtn")
    public void onApproveBtnClick(Button.ClickEvent event) {
        MonitoringCardVersion monitoringCardVersion = versionTable.getSingleSelected();
        if (monitoringCardVersion != null) {
        monitoringCardVersion.setApproved(true);
        dataManager.commit(monitoringCardVersion);
        }
    }
}