package ru.mkb.corp.monmod.web.screens.problemattribute;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.Sort;
import com.haulmont.cuba.gui.Dialogs;
import com.haulmont.cuba.gui.app.core.inputdialog.DialogActions;
import com.haulmont.cuba.gui.app.core.inputdialog.DialogOutcome;
import com.haulmont.cuba.gui.app.core.inputdialog.InputParameter;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.Table;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.model.impl.CollectionContainerSorter;
import com.haulmont.cuba.gui.screen.Install;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import ru.mkb.corp.monmod.entity.attribute.EnumerationItem;

import javax.inject.Inject;
import java.util.Set;

@UiController("mm_EnumerationEditorFragment")
@UiDescriptor("enumeration-editor-fragment.xml")
public class EnumerationEditorFragment extends ScreenFragment {

    @Inject
    protected DataManager dataManager;
    @Inject
    private Dialogs dialogs;

    @Inject
    private CollectionContainer<EnumerationItem> enumerationItemDc;

    @Inject
    private Table<EnumerationItem> enumerationItemTable;

    @Subscribe("enumerationItemTable.add")
    public void onEnumerationItemTableAdd(Action.ActionPerformedEvent event) {
        dialogs.createInputDialog(this)
                .withCaption("Элемент")
                .withParameter(InputParameter.stringParameter("name").withCaption("Наименование").withRequired(true))
                .withActions(DialogActions.OK_CANCEL)
                .withCloseListener(inputDialogCloseEvent -> {
                    if (inputDialogCloseEvent.closedWith(DialogOutcome.OK)) {
                        EnumerationItem e = dataManager.create(EnumerationItem.class);
                        e.setName(inputDialogCloseEvent.getValue("name"));
                        e.setOrd(enumerationItemDc.getMutableItems().size() + 1);
                        enumerationItemDc.getMutableItems().add(e);
                    }
                })
                .show();
    }

    @Subscribe("enumerationItemTable.moveUp")
    public void onEnumerationItemTableMoveUp(Action.ActionPerformedEvent event) {
        Set<EnumerationItem> selected = enumerationItemTable.getSelected();
        if (selected.isEmpty())
            return;
        EnumerationItem currentItem = selected.iterator().next();
        int index = enumerationItemDc.getItemIndex(currentItem);
        if (index == 0)
            return;
        Integer tmp = currentItem.getOrd();
        EnumerationItem prevItem = enumerationItemDc.getMutableItems().get(index - 1);
        currentItem.setOrd(prevItem.getOrd());
        prevItem.setOrd(tmp);
        setSorter();
    }

    @Subscribe("enumerationItemTable.moveDown")
    public void onEnumerationItemTableMoveDown(Action.ActionPerformedEvent event) {
        Set<EnumerationItem> selected = enumerationItemTable.getSelected();
        if (selected.isEmpty())
            return;
        EnumerationItem currentItem = selected.iterator().next();
        int index = enumerationItemDc.getItemIndex(currentItem);
        if (index == enumerationItemDc.getMutableItems().size() - 1)
            return;
        Integer tmp = currentItem.getOrd();
        EnumerationItem nextItem = enumerationItemDc.getMutableItems().get(index + 1);
        currentItem.setOrd(nextItem.getOrd());
        nextItem.setOrd(tmp);
        setSorter();
    }

    @Subscribe
    public void onAfterInit(AfterInitEvent event) {
        setSorter();
    }

    void setSorter() {
        CollectionContainerSorter sorter = new CollectionContainerSorter(enumerationItemDc, null);
        sorter.sort(Sort.by("ord"));
        enumerationItemDc.setSorter(sorter);
    }

    @Install(to = "enumerationItemTable.moveUp", subject = "enabledRule")
    private boolean enumerationItemTableMoveUpEnabledRule() {
        return enumerationItemTable.getSingleSelected() != null && enumerationItemDc.getItemIndex(enumerationItemTable.getSingleSelected()) != 0;
    }

    @Install(to = "enumerationItemTable.moveDown", subject = "enabledRule")
    private boolean enumerationItemTableMoveDownEnabledRule() {
        return enumerationItemTable.getSingleSelected() != null && enumerationItemDc.getItemIndex(enumerationItemTable.getSingleSelected()) != enumerationItemDc.getMutableItems().size() - 1;
    }

    @Subscribe("enumerationItemTable.edit")
    public void onEnumerationItemTableEdit(Action.ActionPerformedEvent event) {
        EnumerationItem singleSelected = enumerationItemTable.getSingleSelected();
        if (singleSelected == null) {
            return;
        }
        dialogs.createInputDialog(this)
                .withCaption("Элемент")
                .withParameter(InputParameter.stringParameter("name").withCaption("Наименование").withRequired(true).withDefaultValue(singleSelected.getName()))
                .withActions(DialogActions.OK_CANCEL)
                .withCloseListener(inputDialogCloseEvent -> {
                    if (inputDialogCloseEvent.closedWith(DialogOutcome.OK)) {
                        singleSelected.setName(inputDialogCloseEvent.getValue("name"));
                    }
                })
                .show();
    }

    @Subscribe(id = "enumerationItemDc", target = Target.DATA_CONTAINER)
    public void onEnumerationItemDcItemPropertyChange(InstanceContainer.ItemPropertyChangeEvent<EnumerationItem> event) {
        if (event.getProperty().equals("byDefault")) {
            if (Boolean.TRUE.equals(event.getValue())) {
                enumerationItemDc.getMutableItems().stream()
                        .filter(enumerationItem -> !enumerationItem.equals(event.getItem()))
                        .forEach(enumerationItem -> {
                            enumerationItem.setByDefault(Boolean.FALSE);
                        });
            }
        }
    }
}