package ru.mkb.corp.monmod.web.screens.problemattribute;

import com.google.common.base.Joiner;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Lists;
import com.google.common.collect.Multimap;
import com.haulmont.chile.core.model.MetaClass;
import com.haulmont.cuba.client.ClientConfig;
import com.haulmont.cuba.core.app.dynamicattributes.PropertyType;
import com.haulmont.cuba.core.entity.*;
import com.haulmont.cuba.core.global.*;
import com.haulmont.cuba.core.sys.xmlparsing.Dom4jTools;
import com.haulmont.cuba.gui.UiComponents;
import com.haulmont.cuba.gui.components.*;
import com.haulmont.cuba.gui.components.data.options.MapOptions;
import com.haulmont.cuba.gui.components.filter.FilterParser;
import com.haulmont.cuba.gui.dynamicattributes.DynamicAttributesGuiTools;
import com.haulmont.cuba.gui.icons.Icons;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.model.InstancePropertyContainer;
import com.haulmont.cuba.gui.screen.*;
import com.haulmont.cuba.gui.sys.ScreensHelper;
import com.haulmont.cuba.gui.theme.ThemeConstants;
import org.apache.commons.lang3.StringUtils;
import ru.mkb.corp.monmod.entity.attribute.EnumerationItem;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttribute;

import javax.inject.Inject;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.haulmont.cuba.core.app.dynamicattributes.PropertyType.*;
import static com.haulmont.cuba.core.entity.CategoryAttributeOptionsLoaderType.*;
import static java.lang.String.format;

@UiController("mm_ProblemAttribute.edit")
@UiDescriptor("problem-attribute-edit.xml")
@EditedEntityContainer("problemAttributeDc")
@LoadDataBeforeShow
public class ProblemAttributeEdit extends StandardEditor<ProblemAttribute> {

    protected static final Multimap<PropertyType, String> FIELDS_VISIBLE_FOR_TYPES = ArrayListMultimap.create();
    protected static final Set<PropertyType> SUPPORTED_OPTIONS_TYPES = ImmutableSet.of(STRING, DOUBLE, DECIMAL, INTEGER, ENTITY);


    static {
        FIELDS_VISIBLE_FOR_TYPES.put(BOOLEAN, "defaultBoolean");
        FIELDS_VISIBLE_FOR_TYPES.put(BOOLEAN, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(STRING, "defaultString");
        FIELDS_VISIBLE_FOR_TYPES.put(STRING, "lookup");
        FIELDS_VISIBLE_FOR_TYPES.put(STRING, "isCollection");
        FIELDS_VISIBLE_FOR_TYPES.put(STRING, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(STRING, "rowsCount");
        FIELDS_VISIBLE_FOR_TYPES.put(STRING, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(DOUBLE, "defaultDouble");
        FIELDS_VISIBLE_FOR_TYPES.put(DOUBLE, "minDouble");
        FIELDS_VISIBLE_FOR_TYPES.put(DOUBLE, "maxDouble");
        FIELDS_VISIBLE_FOR_TYPES.put(DOUBLE, "lookup");
        FIELDS_VISIBLE_FOR_TYPES.put(DOUBLE, "isCollection");
        FIELDS_VISIBLE_FOR_TYPES.put(DOUBLE, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(DOUBLE, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "defaultDecimal");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "minDecimal");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "maxDecimal");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "isCollection");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "numberFormatPattern");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "lookup");
        FIELDS_VISIBLE_FOR_TYPES.put(DECIMAL, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(INTEGER, "defaultInt");
        FIELDS_VISIBLE_FOR_TYPES.put(INTEGER, "minInt");
        FIELDS_VISIBLE_FOR_TYPES.put(INTEGER, "maxInt");
        FIELDS_VISIBLE_FOR_TYPES.put(INTEGER, "lookup");
        FIELDS_VISIBLE_FOR_TYPES.put(INTEGER, "isCollection");
        FIELDS_VISIBLE_FOR_TYPES.put(INTEGER, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(INTEGER, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE, "defaultDate");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE, "defaultDateIsCurrent");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE, "isCollection");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE_WITHOUT_TIME, "defaultDateWithoutTime");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE_WITHOUT_TIME, "defaultDateIsCurrent");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE_WITHOUT_TIME, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE_WITHOUT_TIME, "isCollection");
        FIELDS_VISIBLE_FOR_TYPES.put(DATE_WITHOUT_TIME, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(ENUMERATION, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(ENUMERATION, "tooltip");
        FIELDS_VISIBLE_FOR_TYPES.put(ENUMERATION, "enumerationEditorFragment");
        FIELDS_VISIBLE_FOR_TYPES.put(ENTITY, "entityClass");
        FIELDS_VISIBLE_FOR_TYPES.put(ENTITY, "screen");
        FIELDS_VISIBLE_FOR_TYPES.put(ENTITY, "lookup");
        FIELDS_VISIBLE_FOR_TYPES.put(ENTITY, "defaultEntityId");
        FIELDS_VISIBLE_FOR_TYPES.put(ENTITY, "isCollection");
        FIELDS_VISIBLE_FOR_TYPES.put(ENTITY, "width");
        FIELDS_VISIBLE_FOR_TYPES.put(ENTITY, "tooltip");
    }


    @Inject
    protected Form attributeFieldGroup;
    @Inject
    protected Form optionalAttributeFieldGroup;

    @Inject
    protected Metadata metadata;
    @Inject
    protected MetadataTools metadataTools;
    @Inject
    protected MessageTools messageTools;
    @Inject
    protected DatatypeFormatter datatypeFormatter;
    @Inject
    protected ScreensHelper screensHelper;
    @Inject
    protected DynamicAttributesGuiTools dynamicAttributesGuiTools;
    @Inject
    protected DataManager dataManager;
    @Inject
    protected ReferenceToEntitySupport referenceToEntitySupport;

    @Inject
    private MessageBundle messageBundle;
    @Inject
    private LookupField<Boolean> defaultBoolean;
    @Inject
    private LookupField<String> entityClass;
    @Inject
    private PickerField<Entity> defaultEntityId;
    @Inject
    private InstanceContainer<ProblemAttribute> problemAttributeDc;

    @Inject
    private LookupField<String> screen;
    @Inject
    private InstancePropertyContainer<CategoryAttributeConfiguration> configurationDs;
    @Inject
    private EnumerationEditorFragment enumerationEditorFragment;
    @Inject
    private CollectionContainer<EnumerationItem> enumerationItemDc;


    @Subscribe
    private void init(InitEvent event) {
        initAttributesFieldGroup();
        Action action = new PickerField.LookupAction(defaultEntityId);
        defaultEntityId.addAction(action);
    }

    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {


    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        ProblemAttribute attribute = getEditedEntity();
        CategoryAttributeConfiguration configuration = attribute.getConfiguration();

        if (attribute.getDataType() == ENTITY && Boolean.TRUE.equals(attribute.getLookup()) && configuration.getOptionsLoaderType() == null) {
            attribute.getConfiguration().setOptionsLoaderType(JPQL);
        }
        changeAttributesUI();
        initDsListeners();
        initEnumerationDc();
    }

    private void initEnumerationDc() {
        ProblemAttribute editedEntity = getEditedEntity();
        if (editedEntity.getDataType() != null && editedEntity.getDataType().equals(ENUMERATION)) {
            AtomicInteger atomicInteger = new AtomicInteger();
            enumerationItemDc.setItems(Lists.newArrayList(Splitter.on(",").omitEmptyStrings().split(editedEntity.getEnumeration())).
                    stream()
                    .map(s -> {
                        EnumerationItem item = dataManager.create(EnumerationItem.class);
                        item.setName(s);
                        item.setOrd(atomicInteger.incrementAndGet());
                        item.setByDefault(item.getName().equals(editedEntity.getDefaultString()));
                        return item;
                    })
                    .collect(Collectors.toList())
            );
        }
    }


    @Subscribe(id = "enumerationItemDc", target = Target.DATA_CONTAINER)
    public void onEnumerationItemDcCollectionChange(CollectionContainer.CollectionChangeEvent<EnumerationItem> event) {
        setEnumeration();
    }

    @Subscribe(id = "enumerationItemDc", target = Target.DATA_CONTAINER)
    public void onEnumerationItemDcItemPropertyChange(InstanceContainer.ItemPropertyChangeEvent<EnumerationItem> event) {
        setEnumeration();
    }

    private void setEnumeration() {
        getEditedEntity().setEnumeration(Joiner.on(",")
                .join(enumerationItemDc.getMutableItems()
                        .stream()
                        .sorted(Comparator.comparing(EnumerationItem::getOrd))
                        .map(EnumerationItem::getName)
                        .collect(Collectors.toList())));
        getEditedEntity().setDefaultString(
                enumerationItemDc.getMutableItems()
                        .stream()
                        .filter(EnumerationItem::getByDefault)
                        .map(EnumerationItem::getName)
                        .findFirst()
                        .orElse(null));

    }


    protected void initDsListeners() {
        problemAttributeDc.addItemPropertyChangeListener(e -> {
            String property = e.getProperty();
            if ("dataType".equals(property)
                    || "lookup".equals(property)
                    || "defaultDateIsCurrent".equals(property)
                    || "entityClass".equals(property)) {
                changeAttributesUI();
                changeAttributeValues();
            }
            if ("name".equals(property)) {
                setAttributeCodeValue();
            }
            if ("screen".equals(property) || "joinClause".equals(property) || "whereClause".equals(property)) {
                dynamicAttributesGuiTools.initEntityPickerField(defaultEntityId, e.getItem());
            }
        });
        configurationDs.addItemPropertyChangeListener(e -> {
            getScreenData().getDataContext().setModified(getEditedEntity(), true);
            if ("optionsLoaderType".equals(e.getProperty())) {
                changeAttributesUI();
                changeAttributeValues();
            }
        });
    }

    protected void setAttributeCodeValue() {
        CategoryAttribute attribute = getEditedEntity();
        if (Strings.isNullOrEmpty(attribute.getCode()) && !Strings.isNullOrEmpty(attribute.getName())) {
            String categoryName = StringUtils.EMPTY;
            if (attribute.getCategory() != null) {
                categoryName = StringUtils.defaultString(attribute.getCategory().getName());
            }
            attribute.setCode(StringUtils.deleteWhitespace(categoryName + attribute.getName()));
        }
    }

    protected void changeAttributeValues() {
        CategoryAttribute attribute = getEditedEntity();
        CategoryAttributeConfiguration configuration = attribute.getConfiguration();

        if (attribute.getDataType() == ENTITY) {
            if (!Strings.isNullOrEmpty(attribute.getEntityClass())) {
                Map<String, String> options = ((MapOptions<String>) screen.getOptions()).getItemsCollection();
                attribute.setScreen(options.containsValue(attribute.getScreen()) ? attribute.getScreen() : null);
            }
            if (configuration.getOptionsLoaderType() == SQL) {
                configuration.setOptionsLoaderType(JPQL);
            }
        } else if (configuration.getOptionsLoaderType() == JPQL) {
            configuration.setOptionsLoaderType(null);
        }

        if (attribute.getDataType() == DATE) {
            if (Boolean.TRUE.equals(attribute.getDefaultDateIsCurrent())) {
                attribute.setDefaultDate(null);
            }
        }

        if (attribute.getDataType() == DATE_WITHOUT_TIME) {
            if (Boolean.TRUE.equals(attribute.getDefaultDateIsCurrent())) {
                attribute.setDefaultDateWithoutTime(null);
            }
        }

        if (attribute.getDataType() == BOOLEAN) {
            attribute.setIsCollection(null);
        }

        if (attribute.getDataType() == null || !SUPPORTED_OPTIONS_TYPES.contains(attribute.getDataType())) {
            attribute.setLookup(false);
        }

        if (!Boolean.TRUE.equals(attribute.getLookup())) {
            configuration.setOptionsLoaderType(null);
            configuration.setOptionsLoaderScript(null);
            attribute.setWhereClause(null);
            attribute.setJoinClause(null);
        } else {
            CategoryAttributeOptionsLoaderType optionsType = configuration.getOptionsLoaderType();
            if (optionsType == JPQL) {
                configuration.setOptionsLoaderScript(null);
            } else if (optionsType == GROOVY || optionsType == SQL) {
                attribute.setWhereClause(null);
                attribute.setJoinClause(null);
            } else if (optionsType == null) {
                configuration.setOptionsLoaderScript(null);
                attribute.setWhereClause(null);
                attribute.setJoinClause(null);
                if (attribute.getDataType() == ENTITY) {
                    configuration.setOptionsLoaderType(JPQL);
                }
            }
        }
    }

    protected void changeAttributesUI() {
        ProblemAttribute attribute = getEditedEntity();
        CategoryAttributeConfiguration configuration = attribute.getConfiguration();
        optionalAttributeFieldGroup.setVisible(true);
        for (Component fieldConfig : optionalAttributeFieldGroup.getOwnComponents()) {
            fieldConfig.setVisible(false);
        }

        Collection<String> fields = FIELDS_VISIBLE_FOR_TYPES.get(attribute.getDataType());
        if (fields != null && fields.size() > 0) {
            for (String componentId : fields) {
                optionalAttributeFieldGroup.getComponentNN(componentId).setVisible(true);
            }
            optionalAttributeFieldGroup.setVisible(true);
        }
        if (attribute.getDataType() == ENTITY) {
            if (!Strings.isNullOrEmpty(attribute.getEntityClass())) {
                Class entityClass = attribute.getJavaClassForEntity();
                defaultEntityId.setEditable(true);
                defaultEntityId.setMetaClass(metadata.getClass(entityClass));
                dynamicAttributesGuiTools.initEntityPickerField(defaultEntityId, attribute);
                screen.setOptionsMap(screensHelper.getAvailableBrowserScreens(entityClass));
                setDefaultEntityFieldValue();
            } else {
                defaultEntityId.setEditable(false);
            }
            screen.setVisible(!Boolean.TRUE.equals(attribute.getLookup()));
        }

        if (attribute.getDataType() == DATE) {
            optionalAttributeFieldGroup.getComponentNN("defaultDate").setVisible(!Boolean.TRUE.equals(attribute.getDefaultDateIsCurrent()));
        }

        if (attribute.getDataType() == DATE_WITHOUT_TIME) {
            optionalAttributeFieldGroup.getComponentNN("defaultDateWithoutTime").setVisible(!Boolean.TRUE.equals(attribute.getDefaultDateIsCurrent()));
        }

    }

    protected void setDefaultEntityFieldValue() {
        CategoryAttribute attribute = getEditedEntity();
        MetaClass metaClass = metadata.getClassNN(attribute.getJavaClassForEntity());
        if (attribute.getObjectDefaultEntityId() != null) {
            LoadContext<Entity> lc = new LoadContext<>(attribute.getJavaClassForEntity()).setView(View.MINIMAL);
            String pkName = referenceToEntitySupport.getPrimaryKeyForLoadingEntity(metaClass);
            lc.setQueryString(format("select e from %s e where e.%s = :entityId", metaClass.getName(), pkName))
                    .setParameter("entityId", attribute.getObjectDefaultEntityId());
            Entity entity = dataManager.load(lc);
            if (entity != null) {
                defaultEntityId.setValue(entity);
            } else {
                defaultEntityId.setValue(null);
            }
        }
    }

    protected void initAttributesFieldGroup() {
        defaultBoolean.setOptionsMap(getBooleanOptions());

        entityClass.setOptionsMap(getEntityOptions());

        defaultEntityId.addValueChangeListener(e -> {
            Entity entity = e.getValue();
            if (entity != null) {
                getEditedEntity().setObjectDefaultEntityId(referenceToEntitySupport.getReferenceId(entity));
            } else {
                getEditedEntity().setObjectDefaultEntityId(null);
            }
        });
    }


    protected Map<String, Boolean> getBooleanOptions() {
        Map<String, Boolean> booleanOptions = new TreeMap<>();
        booleanOptions.put(datatypeFormatter.formatBoolean(true), Boolean.TRUE);
        booleanOptions.put(datatypeFormatter.formatBoolean(false), Boolean.FALSE);
        return booleanOptions;
    }

    protected Map<String, String> getEntityOptions() {
        Map<String, String> options = new TreeMap<>();
        for (MetaClass metaClass : metadataTools.getAllPersistentMetaClasses()) {
            if (!metadataTools.isSystemLevel(metaClass)) {
                if (metadata.getTools().hasCompositePrimaryKey(metaClass) && !HasUuid.class.isAssignableFrom(metaClass.getJavaClass())) {
                    continue;
                }
                options.put(messageTools.getDetailedEntityCaption(metaClass), metaClass.getJavaClass().getName());
            }
        }
        return options;
    }

    private String getMessage(String key) {
        return messageBundle.getMessage(key);
    }

    @Subscribe
    public void onValidation(ValidationEvent event) {
        if (getEditedEntity().getDataType().equals(ENUMERATION) && StringUtils.isEmpty(getEditedEntity().getEnumeration())) {
            ValidationErrors errors = new ValidationErrors();
            errors.add(enumerationEditorFragment.getFragment(), "Необходимо заполнить перечисление");
            event.addErrors(errors);
        }
    }
}