package ru.mkb.corp.monmod.web.screens.problemfactorevent;

import com.haulmont.cuba.core.entity.CategoryAttributeValue;
import com.haulmont.cuba.core.global.Scripting;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.CheckBox;
import com.haulmont.cuba.gui.components.DateField;
import com.haulmont.cuba.gui.components.GroupBoxLayout;
import com.haulmont.cuba.gui.components.LookupField;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.model.InstancePropertyContainer;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import groovy.lang.Binding;
import ru.mkb.corp.monmod.entity.attribute.ProblemAttributeHolder;
import ru.mkb.corp.monmod.entity.counterparty.Counterparty;
import ru.mkb.corp.monmod.entity.counterparty.JuridicalClient;
import ru.mkb.corp.monmod.entity.directories.ProblemFactor;
import ru.mkb.corp.monmod.entity.event.CommonAttributesEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEvent;
import ru.mkb.corp.monmod.entity.event.ProblemFactorEventMonitoringFactor;
import ru.mkb.corp.monmod.entity.problemfactorattribute.ProblemFactorAttribute;
import ru.mkb.corp.monmod.web.screens.components.problemattribute.ProblemDynamicAttributesPanel;
import ru.mkb.corp.monmod.web.screens.counterparty.juridicalclient.JuridicalClientEdit;

import javax.inject.Inject;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@UiController("mm_EventProblemFragment")
@UiDescriptor("event-problem-fragment.xml")
public class EventProblemFragment extends ScreenFragment implements ObserveProblemFactor {

    @Inject
    private InstanceContainer<ProblemFactorEventMonitoringFactor> factorDc;
    @Inject
    private InstancePropertyContainer<ProblemAttributeHolder> problemAttributeHolderDc;
    @Inject
    Scripting scripting;
    @Inject
    private CheckBox isIdentifiedField;
    @Inject
    private InstanceContainer<CommonAttributesEvent> eventDc;
    @Inject
    private ProblemDynamicAttributesPanel dynamicAttributesPanel;

    private List<AttributeChangedEventHandler> handlers = new ArrayList<>();
    @Inject
    private DateField<LocalDate> detectedDateField;
    @Inject
    private GroupBoxLayout factorGroupBox;


    public void setInstance(ProblemFactorEventMonitoringFactor problemFactorEventMonitoringFactor) {
        factorGroupBox.setCaption(problemFactorEventMonitoringFactor.getMonitoringFactor().getProblemFactor().getName());
        factorDc.setItem(problemFactorEventMonitoringFactor);
    }

    @Subscribe(id = "problemAttributeHolderDc", target = Target.DATA_CONTAINER)
    public void onProblemAttributeHolderDcItemPropertyChange(InstanceContainer.ItemPropertyChangeEvent<ProblemAttributeHolder> event) {
       detectedProblem();
       final Map<String, CategoryAttributeValue> dynamicAttributes = problemAttributeHolderDc.getItem().getDynamicAttributes();
       handlers.forEach(attributeChangedEventHandler -> attributeChangedEventHandler.onAttributeChanged(createEvent(this,dynamicAttributes.get(event.getProperty().substring(1)))));
    }

    public void detectedProblem(){
        if (factorDc.getItem().getMonitoringFactor().getProblemFactor() != null) {
            final Map<String, CategoryAttributeValue> dynamicAttributes = problemAttributeHolderDc.getItem().getDynamicAttributes();
            String script = factorDc.getItem().getMonitoringFactor().getProblemFactor().getScript();
            Binding binding = new Binding();
            List<ProblemFactorAttribute> pfa = problemAttributeHolderDc.getItem().getFactor().getAttributes();
            for (ProblemFactorAttribute pf : pfa) {
                binding.setVariable(pf.getAttribute().getCode(),
                        Optional.ofNullable(dynamicAttributes.get(pf.getAttribute().getCode()))
                                .map(CategoryAttributeValue::getValue)
                                .orElse(null)
                );
            }
            Boolean b = scripting.evaluateGroovy(script, binding);
            isIdentifiedField.setValue(b);
        }
    }

    private Event createEvent(EventProblemFragment eventProblemFragment, CategoryAttributeValue categoryAttributeValue) {
       Event event = new Event();
       event.setSender(eventProblemFragment);
       event.setAttributeValue(categoryAttributeValue);
       return event;
    }



    @Override
    public void addAttributeChangedEventHandler(AttributeChangedEventHandler event) {
        handlers.add(event);
    }

    @Override
    public void handleAttributeChanged(Event event) {
        List<ProblemFactorAttribute> pfa = problemAttributeHolderDc.getItem().getFactor().getAttributes();
        for (ProblemFactorAttribute attribute : pfa) {
            if (event.getAttributeValue() != null && event.getAttributeValue().getCode().equals(attribute.getAttribute().getCode())) {
                problemAttributeHolderDc.getItem().setValue("+"+event.getAttributeValue().getCode(), event.getAttributeValue().getValue());
            }
        }
    }

    @Subscribe
    public void onInit(InitEvent event) {
        factorGroupBox.setShowAsPanel(true);
        eventCompleted();
    }



    @Subscribe(id = "eventDc", target = Target.DATA_CONTAINER)
    public void onEventDcItemPropertyChange(InstanceContainer.ItemPropertyChangeEvent<CommonAttributesEvent> event) {
        eventCompleted();
    }

    public void eventCompleted() {
        if (eventDc.getItem().getCompleted()) {
            dynamicAttributesPanel.setEnabled(false);
            detectedDateField.setEditable(false);
        } else {
            dynamicAttributesPanel.setEnabled(true);
            detectedDateField.setEditable(true);
        }
    }
}