package ru.mkb.corp.monmod.web.screens.settingsfk;

import com.haulmont.addon.bproc.entity.ProcessDefinitionData;
import com.haulmont.addon.bproc.query.ProcessDefinitionDataQuery;
import com.haulmont.addon.bproc.service.BprocRepositoryService;
import com.haulmont.addon.bproc.web.services.TenantUiHelper;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.LoadContext;
import com.haulmont.cuba.gui.Notifications;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.builders.ScreenBuilder;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.HasValue;
import com.haulmont.cuba.gui.components.LookupField;
import com.haulmont.cuba.gui.components.Table;
import com.haulmont.cuba.gui.components.TextField;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.CollectionPropertyContainer;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.Install;
import com.haulmont.cuba.gui.screen.LoadDataBeforeShow;
import com.haulmont.cuba.gui.screen.OpenMode;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.ScreenFragment;
import com.haulmont.cuba.gui.screen.StandardOutcome;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.Target;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import ru.mkb.corp.monmod.entity.directories.TypicalCovenant;
import ru.mkb.corp.monmod.entity.event.EventType;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFK;
import ru.mkb.corp.monmod.entity.event.settings.SettingsFKItem;
import ru.mkb.corp.monmod.web.screens.typicalcovenant.TypicalCovenantBrowse;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@UiController("mm_SettingsfkFragment")
@UiDescriptor("settingsfk-fragment.xml")
public class SettingsfkFragment extends ScreenFragment {
    @Inject
    private InstanceContainer<SettingsFK> settingsDc;

    @Inject
    private DataManager dataManager;

    @Inject
    private ScreenBuilders screenBuilders;

    @Inject
    private InstanceContainer<EventType> eventTypeDc;
    @Inject
    private CollectionPropertyContainer<SettingsFKItem> itemDc;
    @Inject
    private Table<SettingsFKItem> itemTable;
    @Inject
    private Notifications notifications;

    @Subscribe(id = "eventTypeDc", target = Target.DATA_CONTAINER)
    public void onEventTypeDcItemChange(InstanceContainer.ItemChangeEvent<EventType> event) {
        settingsDc.setItem(dataManager.reload((SettingsFK) event.getItem().getSettings(), settingsDc.getView()));
    }

    @Subscribe("itemTable.create")
    public void onItemTableCreate(Action.ActionPerformedEvent event) {
        var mutableItems = itemDc.getMutableItems();
        var collect = mutableItems.stream().map(SettingsFKItem::getCovenant).collect(Collectors.toList());
        TypicalCovenantBrowse build = screenBuilders.lookup(TypicalCovenant.class, this)
                .withOpenMode(OpenMode.DIALOG)
                .withScreenClass(TypicalCovenantBrowse.class)
                .withSelectValidator(it -> {
                    boolean b = it.getSelectedItems().stream().noneMatch(collect::contains);
                    if (!b) {
                        notifications.create(Notifications.NotificationType.HUMANIZED)
                                .withCaption("Уже добавлен")
                                .show();
                    }
                    return b;
                })
                .withSelectHandler(typicalCovenants -> {
                    mutableItems.addAll(
                            typicalCovenants.stream()
                                    .filter(TypicalCovenant::getIsFin)
                                    .map(typicalCovenant -> {
                                        SettingsFKItem settingsFKItem = getScreenData().getDataContext().create(SettingsFKItem.class);
                                        settingsFKItem.setSettings(settingsDc.getItem());
                                        settingsFKItem.setCovenant(typicalCovenant);
                                        settingsFKItem.setDays(settingsDc.getItem().getDays());
                                        return settingsFKItem;
                                    })
                                    .collect(Collectors.toList()));

                })
                .build();
        build.setLookupComponentMultiSelect(true);
        build.setIsFin(Boolean.TRUE);
        build.show();
    }

    public void setInstance(EventType event) {
        eventTypeDc.setItem(event);
    }

}