package ru.mkb.corp.monmod.web.screens.taskinfo;

import com.haulmont.addon.bproc.entity.ProcessDefinitionData;
import com.haulmont.addon.bproc.entity.TaskData;
import com.haulmont.addon.bproc.entity.UserGroup;
import com.haulmont.addon.bproc.query.TaskDataQuery;
import com.haulmont.addon.bproc.service.BprocRepositoryService;
import com.haulmont.addon.bproc.service.BprocRuntimeService;
import com.haulmont.addon.bproc.service.BprocTaskService;
import com.haulmont.addon.bproc.service.UserGroupService;
import com.haulmont.addon.bproc.web.entities.MyTasksTreeItem;
import com.haulmont.addon.bproc.web.processform.ProcessFormScreens;
import com.haulmont.cuba.core.global.Metadata;
import com.haulmont.cuba.gui.actions.list.EditAction;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.DataGrid;
import com.haulmont.cuba.gui.components.GroupTable;
import com.haulmont.cuba.gui.components.HBoxLayout;
import com.haulmont.cuba.gui.components.Label;
import com.haulmont.cuba.gui.components.Tree;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.screen.Install;
import com.haulmont.cuba.gui.screen.LookupComponent;
import com.haulmont.cuba.gui.screen.MessageBundle;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.StandardLookup;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.security.global.UserSession;

import javax.inject.Inject;
import javax.inject.Named;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

@UiController("mm_MyTasks")
@UiDescriptor("my-tasks.xml")
@LookupComponent("tasksTable")
public class MyTasks extends StandardLookup<TaskData> {


    @Inject
    protected BprocTaskService bprocTaskService;

    @Inject
    protected UserSession userSession;

    @Inject
    protected CollectionContainer<TaskData> tasksDc;

    @Inject
    protected ProcessFormScreens processFormScreens;

    @Inject
    protected UserGroupService userGroupService;


    protected List<TaskData> allGroupTasks;

    protected List<String> userGroupCodes;

    @Subscribe
    protected void onBeforeShow(BeforeShowEvent event) {
        getTasks();

    }

    protected void getTasks() {
        tasksDc.getMutableItems().clear();
            List<TaskData> tasks = getTaskDataList();
            tasksDc.setItems(tasks);

    }

    protected List<TaskData> getTaskDataList() {
        TaskDataQuery candidatesTaskDataQuery = bprocTaskService.createTaskDataQuery();
        if (!getUserGroupCodes().isEmpty()) {
            candidatesTaskDataQuery.taskCandidateGroupIn(getUserGroupCodes());
        }
        candidatesTaskDataQuery.taskCandidateUser(userSession.getCurrentOrSubstitutedUser());
       return allGroupTasks = candidatesTaskDataQuery
                .active()
                .orderByTaskCreateTime()
                .desc()
                .list();
    }


    @Subscribe("tasksTable.edit")
    protected void onTasksTableEdit(Action.ActionPerformedEvent event) {
        TaskData taskData = tasksDc.getItem();
        Screen screen = processFormScreens.createTaskProcessForm(taskData, this);
        screen.addAfterCloseListener(afterCloseEvent -> {
            getTasks();
        });
        screen.show();
    }

    protected List<String> getUserGroupCodes() {
        if (userGroupCodes == null) {
            List<UserGroup> userGroups = userGroupService.getUserGroups(userSession.getCurrentOrSubstitutedUser());
            userGroupCodes = userGroups.stream()
                    .map(UserGroup::getCode)
                    .collect(Collectors.toList());
        }
        return userGroupCodes;
    }
}