package ru.mkb.corp.monmod.web.screens.taskinfo;

import com.haulmont.addon.bproc.entity.TaskData;
import com.haulmont.addon.bproc.web.processform.ProcessFormScreens;
import com.haulmont.cuba.gui.ScreenBuilders;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.HasValue;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.screen.*;
import com.haulmont.cuba.security.global.UserSession;
import ru.mkb.corp.monmod.entity.process.task.TaskInfo;
import ru.mkb.corp.monmod.service.task.TaskService;
import ru.mkb.corp.monmod.service.user.UserService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@UiController("mm_TaskInfo.browse")
@UiDescriptor("task-info-browse.xml")
@LookupComponent("taskInfoesTable")
@LoadDataBeforeShow
public class TaskInfoBrowse extends StandardLookup<TaskInfo> {
    @Inject
    private CollectionLoader<TaskInfo> taskInfoesDl;
    @Inject
    private ProcessFormScreens processFormScreens;
    @Inject
    private CollectionContainer<TaskInfo> taskInfoesDc;
    @Inject
    private ScreenBuilders screenBuilders;
    @Inject
    private TaskService taskService;

    private List<TaskData> allTasks = new ArrayList<>();
    @Inject
    private UserSession userSession;
    @Inject
    private UserService userService;


    @Subscribe("taskInfoesTable.edit")
    protected void onTasksTableEdit(Action.ActionPerformedEvent event) {
        if (taskInfoesDc.getItem().getTaskEnd()) {
           screenBuilders.editor(TaskInfo.class, this)
                   .editEntity(taskInfoesDc.getItem())
                   .withOpenMode(OpenMode.NEW_TAB)
                   .show();
        } else {
        TaskData taskData = allTasks.stream()
                .filter(task -> task.getId().equals(taskInfoesDc.getItem().getTaskId()))
                .findFirst()
                .orElse(null);
        Screen screen = processFormScreens.createTaskProcessForm(taskData, this);
        screen.addAfterCloseListener(afterCloseEvent -> {
            allTasks = taskService.getTaskDataList();
            taskInfoesDl.load();
        });
        screen.show();
        }
    }

    @Subscribe("taskEnd")
    public void onTaskEndValueChange(HasValue.ValueChangeEvent<Boolean> event) {
        if(event.getValue()){
            taskInfoesDl.setParameter("taskEnd", true);
        } else taskInfoesDl.setParameter("taskEnd", false);
            taskInfoesDl.load();
    }

    @Subscribe
    public void onInit(InitEvent event) {
        allTasks = taskService.getTaskDataList();
        taskInfoesDl.setParameter("taskEnd", false);
        if (userService.isLeader(userSession.getCurrentOrSubstitutedUser())) {
            List<UUID> taskIds = new ArrayList<>();
            taskInfoesDc.getMutableItems()
                    .stream()
                    .filter(taskInfo -> taskInfo.getAssignedUser().getGroup().equals(userSession.getCurrentOrSubstitutedUser().getGroup())
                            || taskInfo.getAssignedUser().equals(userSession.getCurrentOrSubstitutedUser()))
                    .forEach(taskInfo -> taskIds.add(taskInfo.getId()));
            if (!taskIds.isEmpty()) {
                taskInfoesDl.setParameter("ids", taskIds);
            }
        } else {
            taskInfoesDl.setParameter("assignedUser", userSession.getCurrentOrSubstitutedUser());
        }
        taskInfoesDl.load();
        taskService.checkExist();
    }
}